<?php
namespace Teknisa\CMP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

class AtualizacaoSolicitacao extends \Teknisa\GER\Controllers\TeknisaCrud 
{
    protected $crudService;
    protected $service;
    protected $request;
    protected $response;
    
    public function __construct($crudService, $parameterBag, $service) 
    {
        $this->crudService = $crudService;
        $this->service = $service;
    }
    
    // Adaptado de 'General.js'
    private function valueToFloat($value) {
        if (empty($value)) return 0.0;
        
		if (is_string($value) && $value) {
			if (strpos($value, ',') === false && strpos($value, '.') === false) {
				return floatval($value);
			} else if (strpos($value, ',') > strpos($value, '.')) {
                $value = str_replace('.', '', $value);
                $value = str_replace(',', '.', $value);
				return floatval($value);
			} else if (strpos($value, ',') < strpos($value, '.')) {
			    $value = str_replace(',', '.', $value);
				return floatval($value);
			} else if (strpos($value, ',') === 0 || strpos($value, '.') === 0) {
			    if (strpos($value, ',') === 0) {
			        $value = str_replace(',', '.', $value);
			    }
				return floatval($value);    
			}
		} else {
			return $value;
		}
    }
    
    private function getNrCompra($row)
    {
        try {
            $dataSourceName = 'getNrCompra';
            $conditions = array(
                ["columnName" => "P_CDFILLANC",  "operator" => "=", "value" => $row['CDFILLANC']],
                ["columnName" => "P_CDFILDEST",  "operator" => "=", "value" => $row['CDFILDEST']],
                ["columnName" => "P_NRAUTOFORN", "operator" => "=", "value" => $row['NRAUTOFORN']],
                ["columnName" => "NRORG",        "operator" => "=", "value" => $this->service->getNrOrg()],
            );
            return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        } catch(\Exception $e) {
           throw $e;
        }
    }
    
    private function getParamGrProdGer($row)
    {
        try {
            $dataSourceName = 'getParamGrProdGer';
            $conditions = array(
                ["columnName" => "P_CDFILDEST",  "operator" => "=", "value" => $row['CDFILDEST']],
                ["columnName" => "NRORG",        "operator" => "=", "value" => $this->service->getNrOrg()],
            );
            return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        } catch(\Exception $e) {
           throw $e;
        }
    }
    
    private function getRelacProdOL($row)
    {
        try {
            $dataSourceName = 'relacprodol';
            $conditions = array(
                ["columnName" => "P_CDPRODUTO",  "operator" => "=", "value" => $row['CDPRODUTO']],
                ["columnName" => "NRORG",        "operator" => "=", "value" => $this->service->getNrOrg()],
            );
            return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        } catch(\Exception $e) {
           throw $e;
        }
    }
    
    private function getItemsolirelacAF($row)
    {
        try {
            $dataSourceName = 'count_itemsolirelacaf';
            $conditions = array(
                ["columnName" => "P_CDFILDEST",    "operator" => "=", "value" => $row['CDFILDEST']],
                ["columnName" => "P_CDFILLANC",    "operator" => "=", "value" => $row['CDFILLANC']],
                ["columnName" => "P_NRAUTOFORN",   "operator" => "=", "value" => $row['NRAUTOFORN']],
                ["columnName" => "P_CDPRODUTO",    "operator" => "=", "value" => $row['CDPRODUTO']],
                ["columnName" => "P_CDMARCA",      "operator" => "=", "value" => $row['CDMARCA']],
                ["columnName" => "P_DTPROGENTAF",  "operator" => "=", "value" => $row['DTPROGENTAF']],
                ["columnName" => "P_NRSEQAPURAF",  "operator" => "=", "value" => $row['NRSEQAPURAF']],
                ["columnName" => "P_CDFILCENTCMP", "operator" => "=", "value" => $row['CDFILCENTCMP']],
                ["columnName" => "NRORG",          "operator" => "=", "value" => $this->service->getNrOrg()]
            );
            return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        } catch(\Exception $e) {
           throw $e;
        }
    }
    
    private function getAtuItemSoli($row, $stProduto, $stMarca)
    {
        try {
            $itemsolirelacaf = $this->getItemsolirelacAF($row);
            $counter = (is_array($itemsolirelacaf) && count($itemsolirelacaf)) ? $itemsolirelacaf[0]['CONTADOR'] : 0;
            $dtProgEntr = empty($row['DTENTRITMSOL']) ? $row['DTPROGENTAF'] : $row['DTENTRITMSOL'];
            
            $conditions = array(
                ["columnName" => "P_CDFILSOLI",    "operator" => "=", "value" => $row['CDFILDEST']],
                ["columnName" => "P_CDPRODUTO",    "operator" => "=", "value" => $stProduto],
                ["columnName" => "P_CDMARCA",      "operator" => "=", "value" => $stMarca],
                ["columnName" => "P_EXTRA",        "operator" => "=", "value" => 'X'],
                ["columnName" => "P_CDPRODUTOOL",  "operator" => "=", "value" => $row['CDPRODUTO']],
                ["columnName" => "P_DTPROGENTR",   "operator" => "=", "value" => $dtProgEntr],
                ["columnName" => "NRORG",          "operator" => "=", "value" => $this->service->getNrOrg()]
            );
            
            if($counter > 0) {
                array_push($conditions,
                    array('columnName' => 'P_NRSEQAPURAF',  'operator' => '=', 'value' => $row['NRSEQAPURAF']),
                    array('columnName' => 'P_CDFILCENTCMP', 'operator' => '=', 'value' => $row['CDFILCENTCMP'])
                );
            } else {
                
            }
    
            if($row['IDFUNCAOFILI'] === 'O') {
                if($counter > 0) {
                    $dataSourceName = 'atuitemsoli_ol_apu';
                } else {
                    $dataSourceName = 'atuitemsoli_ol';
                }
            } else {
                if($counter > 0) {
                    $dataSourceName = 'atuitemsoli_apu';
                } else {
                    $dataSourceName = 'atuitemsoli';
                }
            }
            
            return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
        } catch(\Exception $e) {
           throw $e;
        }
    }
    
    public function updateItemSoli($row, $request, $response) {
        try {
            $this->request = $request;
            $this->response = $response;
            $boCompra = false;
            $nrCompra = $this->getNrCompra($row);
            if(is_array($nrCompra) && count($nrCompra)) {
                $paramGr = $this->getParamGrProdGer($row);
                $boCompra = (is_array($paramGr) && count($paramGr)) ? true : false;
            }
            
            if(!$boCompra) {
                
                $quantDife = $this->valueToFloat($row['QTPROGENTAF']) - $this->valueToFloat($row['QTPROGENTAFORIG']);
                if($quantDife !== 0) {
                    if($row['IDFUNCAOFILI'] === 'O') {
                        $relacprodol = $this->getRelacProdOL($row);
                        if(is_array($relacprodol) && count($relacprodol)) {
                            $stProduto   = $relacprodol[0]['CDPRARVORIG'];
                            $stMarca     = $relacprodol[0]['CDMARCA'];
                        } else {
                            $stProduto   = $row['CDPRODUTO'];
                            $stMarca     = $row['CDMARCA'];
                        }
                    } else {
                        $stProduto   = $row['CDPRODUTO'];
                        $stMarca     = $row['CDMARCA'];
                    }
                    
                    $atuItemSoli = $this->getAtuItemSoli($row, $stProduto, $stMarca);
                    if(is_array($atuItemSoli) && count($atuItemSoli)) {
                        foreach($atuItemSoli as $itemsoli) {
                            $quantDife = $quantDife * $itemsoli['VRFATOCONV'] / $itemsoli['VRFATOSOLI'];
                            while($quantDife !== 0) {
                                if($quantDife > 0) {
                                    $this->service->updateQtSoliProd($itemsoli, $quantDife);
                                    $this->service->updateQtSoliProdApu($row, $itemsoli, $quantDife);
                                    $this->service->insertMovOpSol($row, $itemsoli, $quantDife, 'A');
                                    $quantDife = 0;
                                }
                                
                                if($quantDife < 0) {
                                    if($itemsoli['QTSOLIPROD'] > abs($quantDife)) {
                                         $this->service->updateQtSoliProd($itemsoli, $quantDife);
                                         $this->service->updateQtSoliProdApu($row, $itemsoli, $quantDife);
                                         $this->service->insertMovOpSol($row, $itemsoli, $quantDife, 'A');
                                         $quantDife = 0;
                                    } else {
                                        $this->service->deleteItemSoli($itemsoli);
                                        $this->service->insertMovOpSol($row, $itemsoli, 0, 'E');
                                        $quantDife = $quantDife + $itemsoli['QTSOLIPROD'];
                                    }
                                }           
                            }
                        }
                    }
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
}
