<?php
namespace Teknisa\CMP\Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\CMP\Modules\Cadastros\Services\Cmp04700FornecedorAtendeNaoAtendeService;

class Cmp04700FornecedorAtendeNaoAtendeController extends \Zeedhi\Framework\Controller\Report
{
    protected $connection;
    protected $birtStrategy;
    protected $newCode;
    protected $filiais = array();
    protected $regioes = array();
    protected $errorMessages = array();
    protected $stNomeRel;
    protected $reportMapping = array ();
    
    public function __construct(ReportService $reportService, Cmp04700FornecedorAtendeNaoAtendeService $service, $connection, $birtStrategy, $newCode, $environment)
    {
        parent::__construct($reportService);
        $this->connection    = $connection;
        $this->service       = $service;
        $this->birtStrategy  = $birtStrategy;
        $this->newCode       = $newCode;
        $this->environment   = $environment;
    }
    
    public function generateCode($table)
    {
        return $this->newCode->getCodeByTableName($table, $this->environment->getNrOrg());
    }
    
    public function concatArray($array)
    {
        return join('_', $array);
    }
    
    private function addError($filial, $regiao, $fornecedor)
    {
        if (!isset($this->filiais[$filial])) {
            $this->filiais[$filial] = array();
        }
        
        array_push($this->filiais[$filial], 
            array('REG'=>$regiao,
                  'FOR'=>$fornecedor));
    }
    
    private function decodeErrorMessages()
    {
        $tab = "          ";
        
        foreach ($this->filiais as $filial => $regioes) {
            foreach ($regioes as $regiao) {
                $message = "Unidade: " . ($filial) . ' | ' ." Região: " . ($regiao['REG']) . ' | ' ." Fornecedor: " . ($regiao['FOR']) . "\n";
                array_push($this->errorMessages, $message);
            }
        }
    }
    
    
    
    public function validaPeriodoCalendario($row, $response)
    {
        $fornecedor = $this->service->verifyFornecedor($row);
        
        if(empty($fornecedor)) {
            $cdperiodo = $this->service->verifyCdPeriod($row)[0];
            
            if(!empty($cdperiodo)) {
                if($cdperiodo['IDTIPODIA'] == 'R' && $row['IDUTCALENDCMPGERACAOAF'] == 'S') {
                    $erro = new Error("Periodicidade Recorrente Semanal já prioriza o calendário de compras!", 500);
                    $response->setError($erro);
                } else {
                    $this->service->updatePrCalendar($row);
                }
            } else {
                $erro = new Error("Código de periodicidade não cadastrado!", 500);
                $response->setError($erro);
            }
        } else {
            $erro = new Error("Operação bloqueada! A região da unidade não é atendida pelo fornecedor!", 500);
            $response->setError($erro);
        }
    }
    
    public function saveCodPeri(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();

        $codPeri = $this->service->findCodPeri($row);
        
        foreach($codPeri as $arr) {
            if($arr['CDFORNECED'] == $row['CDFORNECED']) {
                if($arr['CDPERIENTR'] != null) {
                    $this->service->updateCodPeri($row);
                } else {
                    $this->service->insertCodPeri($row);    
                }
            }
        }
        
        $this->validaPeriodoCalendario($row, $response);
    }
    
    public function validaReforfil($row)
    {
        $validations = [];
        $nrlistaForn = self::generateCode('LISTA');
        $nrlistaFili = self::generateCode('LISTA');
        
        if($row['CDFORNECED'] == 'T'){
            $this->service->insertListaTodos($nrlistaForn);
        } else {
            foreach($row['CDFORNECED'] as $cdForneced) {
                $this->service->insertLista($nrlistaForn, $cdForneced, $this->environment->getNrOrg());
            }
        }
        
        if($row['CDREGIAO'] == 'T') {
            if($row['CDFILIAL'] == 'T') {
                $this->service->insertListaTodos($nrlistaFili);
            } else {
                foreach($row['CDFILIAL'] as $cdFilial) {
                    $this->service->insertLista($nrlistaFili, $cdFilial, $this->environment->getNrOrg());
                }
            }
        } else {
            if($row['CDFILIAL'] == 'T') {
                $cdRegiao = self::concatArray($row['CDREGIAO']);
                $this->service->insertListaTodosFili($nrlistaFili, $cdRegiao);
            } else {
                foreach($row['CDFILIAL'] as $cdFilial) {
                    $this->service->insertLista($nrlistaFili, $cdFilial, $this->environment->getNrOrg());
                }
            }
        }
        
        $regNaoAtendida = $this->service->validaReforeg($nrlistaForn, $nrlistaFili);

        foreach ($regNaoAtendida as $regiao) {
            $this->addError($regiao['FILIAL'], $regiao['REGIAO'], $regiao['FORNECEDOR']);
        }
        
        array_push($validations, $regNaoAtendida);
        array_push($validations, $this->service->validaReforfil($nrlistaForn, $nrlistaFili));
        
        $this->service->deleteLista($nrlistaForn);
        $this->service->deleteLista($nrlistaFili);
        return $validations;
    }
    
    public function cadAutoForn(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        if($row['TIPOCAD'] == 'N') {
            $validations = self::validaReforfil($row);
            if(!empty($validations[1])) {
                $response->addDataSet(new DataSet('res', [$validations]));
            } else {
                self::cadAutoFornNAtend($row);
            }
        } else {
            $validations = self::validaReforfil($row);
            if(!empty($validations[1])) {
                $response->addDataSet(new DataSet('res', [$validations]));
            } else {
                self::cadAutoFornAtend($row);
            }
        }
        
        if (!empty($this->filiais)) {
            $this->decodeErrorMessages();
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            throw new \Exception("sheetErrors", 1);
        }
    }
    
    public function cadAuto(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        if($row['TIPOCAD'] == 'N') {
            self::cadAutoFornNAtend($row);
        } else {
            self::cadAutoFornAtend($row);
        }
        
        if (!empty($this->filiais)) {
            $this->decodeErrorMessages();
            $response->addDataSet(new DataSet("errors", $this->errorMessages));
            throw new \Exception("sheetErrors", 1);
        }
    }
    
    public function cadAutoFornNAtend($row)
    {
        $nrlistaForn = self::generateCode('LISTA');
        $nrlistaReg  = self::generateCode('LISTA');
        $nrlistaFili = self::generateCode('LISTA');
        
        if($row['CDFORNECED'] == 'T'){
            $this->service->insertListaTodos($nrlistaForn);
        } else {
            foreach($row['CDFORNECED'] as $cdForneced){
                $this->service->insertLista($nrlistaForn, $cdForneced, $this->environment->getNrOrg());
            }
        }
        
        if($row['CDREGIAO'] == 'T') {
            if($row['CDFILIAL'] == 'T') {
                $this->service->insertListaTodosFili($nrlistaFili);
            } else {
                foreach($row['CDFILIAL'] as $cdFilial) {
                    $this->service->insertLista($nrlistaFili, $cdFilial, $this->environment->getNrOrg());
                }
            }
        } else {
            if($row['CDFILIAL'] == 'T') {
                $cdRegiao = self::concatArray($row['CDREGIAO']);
                $this->service->insertListaTodosFiliReg($nrlistaFili, $cdRegiao);
            } else {
                foreach($row['CDFILIAL'] as $cdFilial) {
                    $this->service->insertLista($nrlistaFili, $cdFilial, $this->environment->getNrOrg());
                }
            }
        }
        
        $this->service->updateReforfil($nrlistaForn, $nrlistaFili);
        $this->service->insertReforfil($nrlistaForn, $nrlistaFili);
        
        $this->service->deleteLista($nrlistaForn);
        $this->service->deleteLista($nrlistaFili);
    }
    
    public function cadAutoFornAtend($row)
    {
        $nrlistaForn = self::generateCode('LISTA');
        $nrlistaReg  = self::generateCode('LISTA');
        $nrlistaFili = self::generateCode('LISTA');
        
        if($row['CDFORNECED'] == 'T') {
            $this->service->insertListaTodos($nrlistaForn);
        } else {
            foreach($row['CDFORNECED'] as $cdForneced) {
                $this->service->insertLista($nrlistaForn, $cdForneced, $this->environment->getNrOrg());
            }
        }
        
        if($row['CDREGIAO'] == 'T') {
            if($row['CDFILIAL'] == 'T') {
                $this->service->insertListaTodosFili($nrlistaFili);
            } else {
                foreach($row['CDFILIAL'] as $cdFilial){
                    $this->service->insertLista($nrlistaFili, $cdFilial, $this->environment->getNrOrg());
                }
            }
        } else {
            if($row['CDFILIAL'] == 'T') {
                $cdRegiao = self::concatArray($row['CDREGIAO']);
                $this->service->insertListaTodosFiliReg($nrlistaFili, $cdRegiao);
            } else {
                foreach($row['CDFILIAL'] as $cdFilial){
                    $this->service->insertLista($nrlistaFili, $cdFilial, $this->environment->getNrOrg());
                }
            }
        }
        
        if(empty($row['CDPERIENTR'])) {
            $this->service->deleteReforfil($nrlistaForn, $nrlistaFili);
        } else {
            $this->service->updateReforfilAtende($nrlistaForn, $nrlistaFili, $row['CDPERIENTR']);
            $this->service->insertReforfilAtende($nrlistaForn, $nrlistaFili, $row['CDPERIENTR']);
        }
        
        $this->service->deleteLista($nrlistaForn);
        $this->service->deleteLista($nrlistaFili);
    }
    
    public function saveFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $filialExists = false;
        
        $fornecedor = $this->service->findFornecedor($row);
        //
        if(!empty($fornecedor)) {
            $erro = new Error("Operação bloqueada! O fornecedor já está cadastrado!", 500);
            $response->setError($erro);
        } else {
            $this->service->insertFornecedor($row);    
        }
    }
    
    public function fornecedorDelete(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        if(!empty($row)) {
            foreach ($row as $value) {
                $fornecedor = $this->service->verifyFornecedor($value);

                if(empty($fornecedor)) {
                    $this->service->deleteFornecedor($value);
                } else {
                    $erro = new Error("Operação bloqueada! A região da unidade não é atendida pelo fornecedor!", 500);
                    $response->setError($erro);
                }
            }
        }
    }
    
    public function fornecedorAtendeDelete(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        if(!empty($row)) {
            foreach ($row as $value) {
                $fornCad = $this->service->verifyFornReforfil($value);

                if(empty($fornCad)) {
                    $this->service->deleteFornecedorAtendeIns($value);
                } else {
                    $this->service->deleteFornecedorAtendeUpd($value);
                }
            }
        }
    }
    
    public function exportFornecedor(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $row['CDFILDEST'] = is_array($row['CDFILDEST'])  ? count($row['CDFILDEST']) > 0 ? $row['CDFILDEST']  : 'T' : 'T';
        
        $nrlista = $this->generateCode('LISTA');
        
        if($row['CDFILDEST'] != 'T') {
            foreach ($row['CDFILDEST'] as $cdfildest) {
                $this->service->insertLista($nrlista, $cdfildest, $row['NRORG']);
            }   
        } else {
           $this->service->insertLista($nrlista, $row['CDFILDEST'], $row['NRORG']);
        }
        
        $this->service->procExportFornecedor($nrlista, $row['CDFILIALORIG']);
        $this->service->deleteLista($nrlista);
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $row['CDFILIAL'] = is_array($row['CDFILIAL']) ? !empty($row['CDFILIAL']) ? self::concatArray($row['CDFILIAL']) : 'T' : 'T';
        $row['CDFILIAL1'] = $row['CDFILIAL'];
        $row['CDFILIAL2'] = $row['CDFILIAL'];
        $row['CDFILIAL3'] = $row['CDFILIAL'];
        $row['CDFORNECED'] = is_array($row['CDFORNECED']) ? !empty($row['CDFORNECED']) ? self::concatArray($row['CDFORNECED']) : 'T' : 'T';
        $row['CDFORNECED1'] = $row['CDFORNECED'];
        $row['CDFORNECED2'] = $row['CDFORNECED'];
        $row['CDFORNECED3'] = $row['CDFORNECED'];
        $row['NRORG1'] = $row['NRORG']; 

        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] =
            array (
            'strategy' => $row['STRATEGY'],
            'parameterMapping' =>
            array (
                "CDFILIAL"    => "CDFILIAL",
                "CDFILIAL1"   => "CDFILIAL1",
                "CDFORNECED"  => "CDFORNECED",
                "CDFORNECED1" => "CDFORNECED1",
                "NRORG"       => "NRORG",
                "CDFILIAL2"   => "CDFILIAL2",
                "CDFILIAL3"   => "CDFILIAL3",
                "CDFORNECED2" => "CDFORNECED2",
                "CDFORNECED3" => "CDFORNECED3",
                "NRORG1"      => "NRORG1"
            ),
        );
    }
}
