<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use \Setforms2\Shared\ORM\DateTime;
use Exception;
use Teknisa\CMP\Util\CmpUtilities;

class Cmp03900LancamentoAutorizacaoDeFornecimentoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $crudService;
    protected $service;
    protected $cmpUtilities;
    protected $fechamentoAF;
    protected $atuSolicita;
    protected $environment;
    protected $inconsistenciasAF;
    
    public function __construct($crudService, $parameterBag, $service, $fechamentoAF, $atuSolicita, CmpUtilities $cmpUtilities, $environment, $inconsistenciasAF) {
        $this->crudService  = $crudService;
        $this->service      = $service;
        $this->cmpUtilities = $cmpUtilities;
        $this->fechamentoAF = $fechamentoAF;
        $this->atuSolicita  = $atuSolicita;
        $this->environment  = $environment;
        $this->inconsistenciasAF = $inconsistenciasAF;
    }
    
    public function retornaCookieEmail(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $cookie = $this->cmpUtilities->retornaCookieEmail($row['NOME_COOKIE']);
        
        $response->addMessage(new Response\Message($cookie));
    }
    
    public function validacoesIniciais(DTO\Request\Row $request, DTO\Response $response) 
    {
        $permissoesOperador         = $this->service->permissoesOperador();
        $centralCompras             = $this->service->centralCompras();
        $parasupri                  = $this->service->parasupri($centralCompras->getRows()[0]["CDFILCENTCMP"]);
        $permissoesGeraisOperador   = $this->service->permissoesGeraisOperador();
        $permissoesFinaisOperador   = $this->service->permExclusaoEdicaoAF($permissoesGeraisOperador);
        $response->addDataSet($permissoesOperador);
        $response->addDataSet($centralCompras);
        $response->addDataSet($parasupri);
        $response->addDataSet(new DataSet("permissoesGeraisOperador", $permissoesGeraisOperador));
        
        if(!empty($permissoesFinaisOperador)) {
            $response->addDataSet(new DataSet("permExclusaoEdicaoAF", $permissoesFinaisOperador));
        }
    }
    
    private function getUtilizaIndiceFinanceiro() 
    {
        $centralCompras = $this->cmpUtilities->getCentralCompras()['CDFILCENTCMP'];
        $parametroIndiceFinanceiro = $this->service->parasupri($centralCompras)->getRows()[0]['IDUTINDICECOL'];
        
        return $parametroIndiceFinanceiro == 'S';
    }
    
    private function getItemAutF($row)
    {
        $dataSourceName = 'itemautf';
        $conditions = array(
            ["columnName" => "CDFILDEST",  "operator" => "=", "value" => $row['CDFILDEST']],
            ["columnName" => "CDFILLANC",  "operator" => "=", "value" => $row['CDFILLANC']],
            ["columnName" => "NRAUTOFORN", "operator" => "=", "value" => $row['NRAUTOFORN']]
        );
        return $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
    }
    
    public function salvarProduto(DTO\Request $request, DTO\Response $response) {
        try {
            $row     = $request->getDataSet()->getRows()[0];
            $oldRow  = $request->getDataSet()->getRows()[1];
            $dadosAF = $request->getDataSet()->getRows()[2];
            
            $utilizaIndiceFinanceiro = $this->getUtilizaIndiceFinanceiro();
            
            $this->dataSourceName = "itemautf";
            
            $row = is_array($row) ? $row : (array)$row;
            
            if(in_array('UPDATE_ALL_ITENS', array_keys($row))) {
                if($row['BLOQ_MULT_DATAS'] === 'S' && $row['UPDATE_ALL_ITENS'] === 'S') {
                    foreach($this->getItemAutF($row) as $item) {
                        $this->service->excluiProdutosDiferentes($item);
                        
                        $item['DTPROGENTAF'] = $row['DTPROGENTAF'];
                        $item['__is_new'] = true;
                                
                        $requestNewSave = new DTO\Request\Row($item, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                        parent::save($requestNewSave, $response);
                    }
                }
            }
            
            $row["IDBAIXITAFS"]     = $row["IDBAIXITAFS"] == 0 ? null : $row["IDBAIXITAFS"];
            $row["IDBAIXITMAF"]     = $row["IDBAIXITMAF"] == 0 ? null : $row["IDBAIXITMAF"];
            $row["QTVOLUITEMAF"]    = $this->valueToFloat($row["QTVOLUITEMAF"]);
            $row["QTPROGENTAF"]     = $this->valueToFloat($row["QTPROGENTAF"]);
            $row["VRDESCITMAF"]     = $this->valueToFloat($row["VRDESCITMAF"]);
            $row["PRECOTOTAL"]      = $this->valueToFloat($row["PRECOTOTAL"]);
            $row["VRPERDESAF"]      = $this->valueToFloat($row["VRPERDESAF"]);
            $row["VRPREUNIAF"]      = $this->getPrecoUnitario($dadosAF, $row, $utilizaIndiceFinanceiro);
            $row["VRINDICEITMAF"]   = $this->getPrecoFinanceiro($row, $utilizaIndiceFinanceiro);
            $row["IDATIVO"]         = "S";
            $row["IDBAIXITMFUTURA"] = "X";
            $row["IDITMSEPARADO"]   = "N";
            $row["DTPROGENTAF"]     = $row["DTPROGENTAF"]." 00:00:00";
            $row["__is_new"]        = true;
            
            if(sizeof($oldRow) > 0)
                $row["__is_new"] = false;
                //$this->service->excluiProdutosDiferentes($oldRow);
            //$this->service->excluiProdutosDiferentes($row);*/
                
            $requestSave = new DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            $retorno = $this->service->CONSISTE_ORCAMENTO($row);
            
            if(isset($row['JUSTIFICATIVAS'])) {
                foreach($row['JUSTIFICATIVAS'] as $justRow) {
                    $this->service->inserirJustificativa($justRow);
                }
            }
            
            try {
                $erroFecha = $this->fechamentoAF->fechaAF($row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN'], $row['CDFILCENTCMP']);
            } catch(\Exception $e) {
                $erroFecha = $e;
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarImposto(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet()->getRows();
            foreach($dataSet as $row) {
                $row["VRIMPITMAF"]  = $this->valueToFloat($row["VRIMPITMAF"]);
                $row["VRALIQITMAF"] = $this->valueToFloat($row["VRALIQITMAF"]);
            }
            $this->dataSourceName = "af_impostos";
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function getFormaPagamento($row) {
        $newArray = is_array($row) ? $row : $row->getArrayCopy();
        
        return in_array('IDTPFORPGTFO', array_keys($newArray)) ? $row['IDTPFORPGTFO'] : $row['IDTPFORPGTAF'];
    }
    
    public function salvarFormaPagamento(DTO\Request $request, DTO\Response $response) {
        try {
            $dadosAF        = $request->getDataSet()->getRows()[0];
            $pgto           = $request->getDataSet()->getRows()[1];
            $prazos         = $request->getDataSet()->getRows()[2];
            $formaPagamento = self::getFormaPagamento($pgto);
            switch($formaPagamento) {
                default:
                    $dadosAF["IDTPFORPGTAF"] = null;
                    break;
                case "F":
                    $dadosAF["IDTPFORPGTAF"] = $formaPagamento;
                    $dadosAF["NRPARCPGTOAF"] = $pgto["NRPARCPGTOAF"];
                    $dadosAF["NRDIAPGFIXAF"] = $pgto["NRDIAPGFIXAF"];
                    $dadosAF["IDUTDIAPGTAF"] = $pgto["IDUTDIAPGTAF"];
                    break;
                case "V":
                    $dadosAF["IDTPFORPGTAF"] = $formaPagamento;
                    $dadosAF["CDFORMPGTO"]   = $pgto["CDFORMPGTO"];
                    $this->salvarPrazoPercentual($request, $response, $prazos);
                    break;
            }
            
            $this->dataSourceName = "autoforne";
            $requestSave = new DTO\Request\Row($dadosAF, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function getPercentualPrazo($row) {
        return in_array('VRPERCPGTOFO', array_keys($row)) ? $row['VRPERCPGTOFO'] : $row['VRPERCPGTOAF'];
    }
    
    public function getDiasPrazo($row) {
        return in_array('NRDIASPGTOFO', array_keys($row)) ? $row['NRDIASPGTOFO'] : $row['NRDIASPGTOAF'];
    }
    
    public function salvarPrazoPercentual(DTO\Request $request, DTO\Response $response, $prazos) {
        try {
            if(count($prazos)) {
                // Exclui registros antigos e salva os novos prazos cadastrados
                $this->service->removePrazoPercentual($prazos[0]);
                
                for($i = 0; $i < count($prazos); $i++) {
                    $prazos[$i]["VRPERCPGTOAF"] = self::getPercentualPrazo($prazos[$i]);
                    $prazos[$i]["NRDIASPGTOAF"] = self::getDiasPrazo($prazos[$i]);
                    
                    if(isset($prazos[$i]["VRPERCPGTOAF"])){
                        $prazos[$i]["VRPERCPGTOAF"] = $this->valueToFloat($prazos[$i]["VRPERCPGTOAF"]);
                    }
                    
                    $prazos[$i]["__is_new"] = true;
                }
                
                $this->dataSourceName = "prazopgtaf";
                $dataSave = new DataSet('prazopgtaf', (array)$prazos);
                $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($requestSave, $response);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarEncargo(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSet  = $request->getDataSet()->getRows();
            if(sizeof($dataSet) > 0) {
                foreach($dataSet as $row) {
                    $row["VRENCARGOAF"]  = $this->valueToFloat($row["VRENCARGOAF"]);
                    $row["DTENTRENCAF"]  = $row["DTPROGENTAF"]." 00:00:00";
                    if(!$this->service->existeEncargoAF($row)) {
                        $row["__is_new"] = true;
                    }
                }
                
                $this->dataSourceName = "encargoaf";
                parent::save($request, $response);
                $this->service->excluirEncargoAFVazio($dataSet[0]);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function editarJustificativa(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->editarJustificativa($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function excluirProduto(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSet  = $request->getDataSet()->getRows();
            $newDataset = array();
            $messageCode = '';
            
            foreach($dataSet as $key => $produto) {
                if(empty($produto['IDBAIXITMAF']) && $produto['IDAFEXPIRADA'] != 'S' &&
                    $produto['IDBAIXAAF'] != '4' && $produto['IDBAIXAAF'] != '5' &&
                    (empty($produto['IDBAIXAFSAI']) || $produto['IDBAIXAFSAI'] == '1') &&
                    empty($produto['CDFILDESTCON'])) {
                           
                    $produto['__is_new'] = false;
                    array_push($newDataset, $produto);
                }
            }

            if($newDataset) {
                $this->dataSourceName = "itemautf";
                $dataSave = new DataSet("itemautf", (array)$newDataset);
                
                $produtoReferencia1 = array_shift($newDataset);
                $this->service->excluiNFContaPagar($produtoReferencia1['CDFILDEST'], $produtoReferencia1['CDFILLANC'], $produtoReferencia1['NRAUTOFORN']);
                
                $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                
                parent::delete($requestSave, $response);
                
                $bkpDataset = $newDataset;
                $produtoReferencia = array_shift($newDataset);
                
                $itemautf = $this->getItemAutF($produtoReferencia);
                
                $erroFecha = $this->fechamentoAF->fechaAF($produtoReferencia1['CDFILDEST'], $produtoReferencia1['CDFILLANC'], $produtoReferencia1['NRAUTOFORN'], $produtoReferencia1['CDFILCENTCMP']);
                
                //$this->service->fechamentoProcAF($produtoReferencia1['CDFILDEST'], $produtoReferencia1['CDFILLANC'], $produtoReferencia1['NRAUTOFORN']);            
                $this->service->incluiNFContaPagar($produtoReferencia1['CDFILDEST'], $produtoReferencia1['CDFILLANC'], $produtoReferencia1['NRAUTOFORN']);
                
                if(is_array($itemautf) && count($itemautf)) {
                    if(count($bkpDataset) === count($dataSet)) {
                        
                        $messageCode = array('MESSAGE' => 'REG_SUCCESS_DELETE', 'TYPE' => 'SUCCESS');
                    } else {
                        $messageCode = array('MESSAGE' => 'REG_NOTALL_DELETED', 'TYPE' => 'ERROR');    
                    }
                    
                    try {
                        $erroFecha = $this->fechamentoAF->fechaAF($produtoReferencia['CDFILDEST'], $produtoReferencia['CDFILLANC'], $produtoReferencia['NRAUTOFORN'], $produtoReferencia['CDFILCENTCMP']);
                    } catch(\Exception $e) {
                        $erroFecha = $e;
                    }
                } else {
                    //Caso todos os itens sejam excluídos a AF também será excluída
                    $this->service->excluiNFContaPagar($produtoReferencia['CDFILDEST'], $produtoReferencia['CDFILLANC'], $produtoReferencia['NRAUTOFORN']);
                    $this->service->excluirAF($produtoReferencia);
                    $messageCode = array('MESSAGE' => 'DELETED_AF_WITHOUT_ITEMS', 'TYPE' => 'ERROR');
                }
            } else {
                
                $messageCode = array('MESSAGE' => 'ERROR_DELETING_ITEMS', 'TYPE' => 'ERROR');
            }
            
            $response->addMessage(new Message($messageCode, 'W'));
            //$response->addDataSet(new DataSet('produtos', $dataSet));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
        
    }
    
    public function validaFornecedor(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $row["IDULTIDTVALQ"] = $this->service->retornaParametrosCompras($row["CDFILDEST"])[0]["IDULTIDTVALQ"];
            
            if($this->service->fornecedor_qualificacao($row) == 0) {
                throw new Exception("FORNECEDOR_ERROR_QUALIFICACAO");
            } else if($this->service->fornecedor_filial($row) > 0) {
                throw new Exception("FORNECEDOR_NAO_ATENDE_UNIDADE");
            } else if($this->service->fornecedor_regiao($row) > 0) {
                throw new Exception("FORNECEDOR_NAO_ATENDE_REGIAO");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function insereImpostos(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $impostos = $this->service->verificaImposto($row);
            if(sizeof($impostos) > 0) {
                $row["CDINSCESTA"] = $impostos[0]["CDINSCESTA"];
                $this->service->IMPOSTO_ITEMAF($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        } 
    }
    
    public function exigeLiberacaoAFManual($dadosAF)
    {
        if ($dadosAF["IDTIPOLANCAF"] != "M") {
            return false;
        }
        
        $paramCmpSup = $this->service->parcmpsup($dadosAF)[0];
        
        return $paramCmpSup["IDOBLIBAFMAN"] == "S";
    }
    
    public function atualizarInconsistencias(DTO\Request $request, DTO\Response $response) {
        try {
            $limiteParam   = $request->getDataSet()->getRows()[0];
            $inconsisParam = $request->getDataSet()->getRows()[1];
            $procedure     = $request->getDataSet()->getRows()[2][0];
            $dadosAF       = $request->getDataSet()->getRows()[3];
            
            if(isset($dadosAF["VRTOTALAF"]))
                $dadosAF["VRTOTALAF"] = $this->valueToFloat($dadosAF["VRTOTALAF"]);
            
            if(sizeof($limiteParam) > 0) {
                $dadosAF["IDAFFORALIM"] = "S";
                $dadosAF["CDGERENTE"]   = "";
            } else {
                $dadosAF["IDAFFORALIM"] = "N";
            }
            
            if(sizeof($inconsisParam) > 0 || self::exigeLiberacaoAFManual($dadosAF)) {
                $dadosAF["IDSITUATUAAF"] = "1";
            } else {
                $dadosAF["IDSITUATUAAF"] = "";
            }
            
            // Salva AF
            $this->dataSourceName = "autoforne";
            $requestSave = new DTO\Request\Row($dadosAF, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            // Atualiza valor Total
            $this->service->valorTotalAF($dadosAF);
            
            // Atualiza Contar a Pagar
            //$tpctpagar = $this->service->updateCdtpctpagar($dadosAF);
            $tpctpagar   = "";
            if ($tpctpagar) {
                $tpctpagar = $this->service->retornaCdtpctpagar($dadosAF);
                $tpctpagar = array($tpctpagar[0]["CDTPCTPAGAR"], $tpctpagar[0]["DSTPCTPAGAR"]);
            } else {
                $tpctpagar = array();
            }
            
            // Procedure CUSTO_AF
            if($procedure) {
                $dadosAF["NRSEQAPURAF"] = null;
                $this->service->CUSTO_AF($dadosAF);
            }
            
            //$response->addDataSet(new DataSet('updateCdtpctpagar', $tpctpagar));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function consisteOrcamento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $retorno = $this->service->CONSISTE_ORCAMENTO($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function consisteOrcamentoGopWorkflow(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            self::consisteOrcamentoGop($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function consisteDsincons(DTO\Request $request, DTO\Response $response) {
        try {
            $row          = $request->getDataSet()->getRows()[0];
            $produtos     = $request->getDataSet()->getRows()[1];
            $dsinconsitaf = $request->getDataSet()->getRows()[2];
            
            if(sizeof($dsinconsitaf) > 0) {
                $row["DSINCONSAF"] = $dsinconsitaf[0];
                $this->dataSourceName = "autoforne";
                $requestSave = new DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($requestSave, $response);
            }
            
            $this->dataSourceName = "itemautf";
            $dataSave = new DataSet('itemautf', (array)$produtos);
            $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            $inconsistencia = $this->inconsistenciasAF->verificaInconsistencias($row["CDFILDEST"], $row["CDFILLANC"], $row["NRAUTOFORN"], $row["CDFILCENTCMP"], true);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function isAutorizacaoFornecimentoEmAberto($dadosAF)
    {
        return !$dadosAF["IDBAIXAAF"];
    }
    
    public function isAutorizacaoFornecimentoExpirada($dadosAF)
    {
        return isset($dadosAF["IDAFEXPIRADA"]) && $dadosAF["IDAFEXPIRADA"] == 'S';
    }
    
    public function atualizaProdutosTipoBaixa(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $dadosAF  = $request->getRow();
            $produtos = $this->service->retornaProdutos($dadosAF);
            $isChanged = false;
            $itemJaBaixado = 0;
            
            for($i = 0; $i < sizeof($produtos); $i++) {
                $idbaixitafs = $dadosAF["VALOR_IDBAIXITMAF"];
                if($produtos[$i]["IDBAIXITMAF"] != "2" && $produtos[$i]["IDBAIXITAFS"] != "2") {
                    if(($produtos[$i]["QTEFETENTRAF"] > 0) && ($idbaixitafs == null)){
                       $idbaixitafs = "1";
                       $itemJaBaixado++;
                    } 
                    $produtos[$i]["IDBAIXITMAF"] = $idbaixitafs;
                    $produtos[$i]["changed"] = true;
                    $isChanged = true;
                } else {
                    $produtos[$i]["changed"] = false;
                    $itemJaBaixado++;
                }
                if($dadosAF["RETORNAR_ABERTO"]) {
                    $produtos[$i]["DTBAIXITMAF"] = null;
                    $produtos[$i]["CDOPERBAIXA"] = null;
                }
                if($dadosAF["BAIXAR_TOTALMENTE"]) {
                    $produtos[$i]["DTBAIXITMAF"] = $produtos[$i]["DTPROCESSA"];
                    $produtos[$i]["CDOPERBAIXA"] = $this->service->operadorLogado();
                }
                
                $produtos[$i]["DTPROGENTAF"] = $produtos[$i]["DTPROGENTAF"] ? $produtos[$i]["DTPROGENTAF"]." 00:00:00" : null;
                $produtos[$i]["IDBAIXITMAF"] = $produtos[$i]["IDBAIXITMAF"] ? $produtos[$i]["IDBAIXITMAF"] : null;
                $produtos[$i]["IDBAIXITAFS"] = $produtos[$i]["IDBAIXITAFS"] ? $produtos[$i]["IDBAIXITAFS"] : null;
            }
            
            $this->dataSourceName = "itemautf";
            $dataSave = new DataSet('itemautf', (array)$produtos);
            $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            if($isChanged) {
                if (self::isAutorizacaoFornecimentoEmAberto($dadosAF)) {
                    $dadosAF["IDBAIXAAF"] = null;
                    if ($itemJaBaixado > 0){
                        $dadosAF["IDBAIXAAF"] = "1";    
                    }
                    if (self::isAutorizacaoFornecimentoExpirada($dadosAF)) {
                        $dadosAF["IDAFEXPIRADA"] = 'N';
                    }
                }
                
                $this->dataSourceName = "autoforne";
                parent::save($request, $response);
            }
            
            if ($dadosAF['IDBAIXAAF'] == '4' || $dadosAF['IDBAIXAAF'] == '5') {
                if (!empty($dadosAF['COOKIE_NAME'])) {
                    self::enviarEmailFornecedor($dadosAF);
                }
            }
            self::consisteOrcamentoGop($dadosAF);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    function enviarEmailFornecedor($dadosAF) 
    {
        $this->cmpUtilities->defineCookieEmail($dadosAF['COOKIE_NAME'], $dadosAF['EMAIL_MENSAGEM']);
        
        $params = array("[N° AF]", "[Nome Operador]");
        $values = array($dadosAF['NRAUTOFORN'], $dadosAF['NMOPERADOR']);
        
        $message = str_replace($params, $values, $dadosAF['EMAIL_MENSAGEM']);

        $listaEmailFornecedor = $this->service->retornaEmailFornecedor($dadosAF['CDFORNECED']);
        
        if (count($listaEmailFornecedor)) {
            $emails = array_map(function($emailFornecedor) {
                return $emailFornecedor['DSLOGINECO'];
            }, $listaEmailFornecedor);
            
            if(is_array($emails) && count($emails)) {
                $emails = explode(";", $emails[0]);    
            }
            
            $messageSendMail = $this->cmpUtilities->messageSendMail($dadosAF['FILIAL_LOGADA'], $dadosAF['EMAIL_ASSUNTO'], $message, 'PORTAL E-COMMERCE');
            
            try{
                return $this->cmpUtilities->enviaEmail('', $dadosAF['EMAIL_ASSUNTO'], $emails, $messageSendMail, $emails);
            } catch (\Exception $e) {
                return false;
            }
            
        }
    }
    
    public function atualizaItemBaixa(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet()->getRows();
            $newDataset = array();
            
            foreach($dataSet as $key => $produto) {
                if($produto['IDBAIXITMAF'] !== '2' && $produto['IDBAIXITMAF'] !== '3' &&
                   $produto['IDITMSEPARADO'] !== 'S' &&
                   $produto['IDBAIXAAF'] != '4' && $produto['IDBAIXAAF'] != '5' &&
                   (empty($produto['IDBAIXAFSAI']) || $produto['IDBAIXAFSAI'] == '1') &&
                   ($produto['IDAFOPERLOG'] != '1' || length($produto['CDFILIOPELOG']) > 0) &&
                   $produto['IDAFEXPIRADA'] != 'S') {
                             
                    $idbaixitafs = $produto["VALOR_IDBAIXITMAF"];
                
                    if ($produto["IDBAIXITMAF"] != "2" && $produto["IDBAIXITAFS"] != "2") {
                        $produto["IDBAIXITMAF"] = $idbaixitafs;
                        $produto["changed"] = true;
                    } else {
                        $produto["changed"] = false;
                    }
                    
                    $produto["QTPROGENTAF"] = $this->valueToFloat($produto["QTPROGENTAF"]);
                    $produto["VRDESCITMAF"] = $this->valueToFloat($produto["VRDESCITMAF"]);
                    $produto["VRPREUNIAF"]  = $this->valueToFloat($produto["VRPREUNIAF"]);
                    
                    $produto["DTPROGENTAF"] = $produto["DTPROGENTAF"] ? $produto["DTPROGENTAF"]." 00:00:00" : null;
                    $produto["IDBAIXITMAF"] = $produto["IDBAIXITMAF"] ? $produto["IDBAIXITMAF"] : null;
                    $produto["IDBAIXITAFS"] = $produto["IDBAIXITAFS"] ? $produto["IDBAIXITAFS"] : null;
                    $produto["DTBAIXITMAF"] = $produto["DTPROCESSA"];
                    $produto["CDOPERBAIXA"] = $this->service->operadorLogado();
                    
                    array_push($newDataset, $produto);
                } else {
                    unset($dataSet[$key]);
                }
            }
            
            if($newDataset) {
                $this->dataSourceName = "itemautf";
                $dataSave = new DataSet("itemautf", (array)$newDataset);
                $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($requestSave, $response);
                
                $produtoReferencia = array_shift($newDataset);

                self::atualizaTipoBaixaAF($produtoReferencia);
            
                self::consisteOrcamentoGop($produtoReferencia);
            }
            
            $response->addDataSet(new DataSet('produtos', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function atualizaTipoBaixaAF($row)
    {
        $qtdeProdutos         = (float) $this->service->getCountProdutos($row)['QTDE'];
        $qtdeProdutosBaixados = (float) $this->service->getCountProdutosBaixados($row)['QTDE'];
        
        $row['IDBAIXAAF'] = $qtdeProdutos == $qtdeProdutosBaixados ? '3' : '1';
        
        $this->service->updateBaixaAf($row);
    }
    
    public function salvarJustificativa(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->inserirJustificativa($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarJustificativas(DTO\Request $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet()->getRows();
            foreach($dataSet as $row) {
                $this->service->inserirJustificativa($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaFilial(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $dataset = [];
            $cdfilial = $request->getRow()["CDFILIAL"];
            
            if($this->service->parasupri_filial_validacao($cdfilial) != false) {
                $idtipemitger = $this->service->parcmpsup_validacao($cdfilial);
                
                if($idtipemitger != false) {
                    $dataset["IDTIPEMITGER"] = $idtipemitger;
                    $dataset["DTPROCESSA"] = self::retornaDtprocessa($cdfilial);
                    
                    $response->addDataSet(new DataSet('dados', $dataset));
                } else {
                    throw new Exception("Unidade não possui parametrização de compras e suprimentos.");
                }
            } else {
                throw new Exception("Unidade não possui central de compras.");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaComprador(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $op = $this->crudService->getFilteredDataSet("comproper_comprador", $conditions)->getRows();
            $dataset = [];
            
            if(sizeof($op) > 0) {
                if($op[0]["IDAFCMPOPER"] != "N") {
                    if($op[0]["IDEXSENHCMPOPER"] == "S")
                        $dataset["EXIGE_SENHA"] = true;
                    $response->addDataSet(new DataSet('dados', $dataset));
                } else {
                    throw new Exception("Operador não autorizado a incluir AF's deste comprador.");
                }
            } else {
                throw new Exception("Operador não relacionado ao comprador.");
            }
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function confirmaSenhaComprador(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $senha = $this->service->senhaOperador($row["CDCOMPRADOR"]);
            if($senha && md5($row["SENHA"]) != $senha)
                throw new Exception("Senha inválida!");
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function retornaDadosFornecedor(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $fornecedor = $this->service->fornecedor($row);
            if($fornecedor["IDSITUCADA"] != "I") {
                if(isset($row["CDTIPCOB"]) && $row["CDTIPCOB"])
                    $fornecedor["NMTIPOCOBR"] = $this->service->retornaTipoCobranca($row["CDTIPCOB"]);
                else
                    $fornecedor["NMTIPOCOBR"] = "";
                $response->addDataSet(new DataSet('fornecedor', $fornecedor));
            } else {
                throw new Exception("Fornecedor Inativo.");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaDataLancamento(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $dtProcessa = self::retornaDtprocessa($row["CDFILDEST"]);
            $IDVALDTAF  = $this->service->retornaParametrosCompras($row["CDFILDEST"])[0]["IDVALDTAF"];
            
            $dtProcessamento = DateTime::createFromFormat("d/m/Y", $dtProcessa);
            $dtLancamento    = DateTime::createFromFormat("d/m/Y", substr($row["DTAUTOFORN"], 0, 10));
            
            if(empty($dtProcessamento)) {
                $dataAtual = DateTime::createFromFormat("d/m/Y", date('d/m/Y'));
                if($dtLancamento->format("d/m/Y") !== $dataAtual->format("d/m/Y")) {
                    throw new Exception("Data de lançamento deve ser igual à data atual.");
                }
            } else {
                if($IDVALDTAF == "D" && $dtProcessamento->format("d/m/Y") != $dtLancamento->format("d/m/Y")) {
                    throw new Exception("Data de lançamento deve ser igual à data de processamento " . $dtProcessamento->format("d/m/Y") . ".");
                } else if($IDVALDTAF == "M" && $dtProcessamento->format("m") != $dtLancamento->format("m")) {
                    throw new Exception("Mês de lançamento deve ser menor ou igual ao mês de processamento " . $dtProcessamento->format("d/m/Y") . ".");
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    } 
    
    public function salvarAdicaoAF(DTO\Request $request, DTO\Response $response) 
    {
        try {
            $dadosAF  = $request->getDataSet()->getRows()[0];
            $produtos = $request->getDataSet()->getRows()[1];
            $mensagem = $request->getDataSet()->getRows()[2];
            $formaPg  = $request->getDataSet()->getRows()[3];
            $encargo  = $request->getDataSet()->getRows()[4];
            $justif   = $request->getDataSet()->getRows()[5];
            $prazos   = $request->getDataSet()->getRows()[6];
            $anexos   = $request->getDataSet()->getRows()[7];
            
            if(!isset($dadosAF["NRAUTOFORN"])) {
                $dadosAF["NRAUTOFORN"]   = $this->service->novoCodigo($dadosAF["CDFILDEST"], $dadosAF["CDFILLANC"]);
                $dadosAF["IDAFFORALIM"]  = "N";
                $dadosAF["IDTIPOLANCAF"] = "M";
                $dadosAF["IDBAIXAAF"]    = null;
                $dadosAF["IDBAIXAFSAI"]  = null;
                $dadosAF["IDSITUATUAAF"] = null;
                $dadosAF["__is_new"]     = true;
            } else {
                $dadosAF["IDBAIXAAF"]   = isset($dadosAF["IDBAIXAAF"])   && $dadosAF["IDBAIXAAF"]   ? $dadosAF["IDBAIXAAF"]   : null;
                $dadosAF["IDBAIXAFSAI"] = isset($dadosAF["IDBAIXAFSAI"]) && $dadosAF["IDBAIXAFSAI"] ? $dadosAF["IDBAIXAFSAI"] : null;
                $dadosAF["IDAFOPERLOG"] = isset($dadosAF["IDBAIXAFSAI"]) && $dadosAF["IDAFOPERLOG"] != "N" ? $dadosAF["IDAFOPERLOG"] : null;
                $dadosAF["__is_new"] = false;
            }
            
            $this->service->excluiTodosProdutos($dadosAF);
            $this->dadosFormaPagamento($dadosAF);
            if(isset($dadosAF["VRTOTALAF"]))
                $dadosAF["VRTOTALAF"]   = $this->valueToFloat($dadosAF["VRTOTALAF"]);
            
            $utilizaIndiceFinanceiro = $this->getUtilizaIndiceFinanceiro();
            
            if ($utilizaIndiceFinanceiro) {
                $dadosAF['CDTPINDICEAF'] = $dadosAF['CDTPINDICE'] ?? $dadosAF['CDTPINDICEAF'];
                $dadosAF['IDPERVALORAF'] = $dadosAF['IDPERVALOR'] ?? $dadosAF['IDPERVALORAF'];
                $dadosAF['CDINDICEAF']   = $dadosAF['CDINDICE'] ?? $dadosAF['CDINDICEAF'];
                $dadosAF['DTREFINDAF']   = $dadosAF['DTREFIND'] ?? $dadosAF['DTREFINDAF'];
            } else {
                $dadosAF['CDTPINDICEAF'] = null;
                $dadosAF['IDPERVALORAF'] = null;
                $dadosAF['CDINDICEAF']   = null;
                $dadosAF['DTREFINDAF']   = null;
            }
            
            $isImport = false;

            foreach($produtos as &$produto) {
                $produto["CDFILLANC"]       = empty($produto["CDFILLANC"]) ? $dadosAF['CDFILLANC'] : $produto["CDFILLANC"];
                $produto["QTVOLUITEMAF"]    = $this->valueToFloat($produto["QTVOLUITEMAF"]);
                $produto["QTPROGENTAF"]     = $this->valueToFloat($produto["QTPROGENTAF"]);
                $produto["VRPREUNIAF"]      = $this->getPrecoUnitario($dadosAF, $produto, $utilizaIndiceFinanceiro);
                $produto["VRINDICEITMAF"]   = $this->getPrecoFinanceiro($produto, $utilizaIndiceFinanceiro);
                $produto["PRECOTOTAL"]      = $this->valueToFloat($produto["PRECOTOTAL"]);
                $produto["VRPERDESAF"]      = $this->valueToFloat($produto["VRPERDESAF"]);
                $produto["VRDESCITMAF"]     = $this->valueToFloat($produto["VRDESCITMAF"]);
                $produto["NRAUTOFORN"]      = $dadosAF["NRAUTOFORN"];
                $produto["CDFORNECED"]      = $dadosAF["CDFORNECED"];
                $produto["CDEMPRESA"]       = $dadosAF["CDEMPRESA"];
                $produto["NRORG"]           = $dadosAF["NRORG"];
                $produto["IDBAIXITAFS"]     = isset($produto["IDBAIXITAFS"]) && $produto["IDBAIXITAFS"] ? $produto["IDBAIXITAFS"] : null;
                $produto["IDBAIXITMAF"]     = isset($produto["IDBAIXITMAF"]) && $produto["IDBAIXITMAF"] ? $produto["IDBAIXITMAF"] : null;
                $produto["IDBAIXITMFUTURA"] = "X";
                $produto["__is_new"]        = true;
                
                if(in_array('IDITMIPORT', array_keys($produto))) {
                    $isImport = $produto['IDITMIPORT'] ? true : false;
                }
            }
            
            $dadosAF["IDTPFORPGTAF"] = self::getFormaPagamento($formaPg);

            if (self::exigeLiberacaoAFManual($dadosAF)) {
                $dadosAF["IDSITUATUAAF"] = "1";
            }
            
            $this->dataSourceName = "autoforne";
            $requestSave = new DTO\Request\Row($dadosAF, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            if($isImport) {
                $this->service->saveImportProducts($produtos);
            } else {
                $this->dataSourceName = "itemautf";
                $dataSave = new DataSet("itemautf", (array)$produtos);
                $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($requestSave, $response);
            }
            
            // Atualiza valor Total
            $this->service->valorTotalAF($dadosAF);
            
            // Verifica se existem inconsistências
            $inconsistencias = $this->service->existeInconsistencias($dadosAF);
            if(sizeof($inconsistencias) == 0) {
                $produto['CDFILLANC'] = empty($produto['CDFILLANC']) ? $dadosAF['CDFILLANC'] : $produto['CDFILLANC'];
                
                // Salva Mensagens
                for($i = 0; $i < sizeof($mensagem); $i++) {
                    $mensagem[$i]["NRAUTOFORN"] = $dadosAF["NRAUTOFORN"];
                    $mensagem[$i]["CDFILDEST"]  = $dadosAF["CDFILDEST"];
                    $mensagem[$i]["CDFILLANC"]  = $dadosAF["CDFILLANC"];
                }
                $this->dataSourceName = "mensautf";
                $dataSave = new DataSet("mensautf", (array)$mensagem);
                $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($requestSave, $response);
                
                // Salva Prazo Percentual
                for($i = 0; $i < sizeof($prazos); $i++) {
                    $prazos[$i]["NRAUTOFORN"] = $dadosAF["NRAUTOFORN"];
                    $prazos[$i]["CDFILDEST"]  = $dadosAF["CDFILDEST"];
                    $prazos[$i]["CDFILLANC"]  = $dadosAF["CDFILLANC"];
                }
                $this->salvarPrazoPercentual($request, $response, $prazos);
                
                // Salva Encargos
                $encargosPreenchidos = [];
                for($i = 0; $i < sizeof($encargo); $i++) {
                    if(isset($encargo[$i]["VRENCARGOAF"]) && $encargo[$i]["VRENCARGOAF"]) {
                        $encargo[$i]["CDFILLANC"]   = $produto["CDFILLANC"];
                        $encargo[$i]["CDFILDEST"]   = $produto["CDFILDEST"];
                        $encargo[$i]["NRAUTOFORN"]  = $produto["NRAUTOFORN"];
                        $encargo[$i]["DTPROGENTAF"] = $produto["DTPROGENTAF"];
                        $encargo[$i]["DTENTRENCAF"] = $produto["DTPROGENTAF"];
                        $encargo[$i]["VRENCARGOAF"] = $this->valueToFloat($encargo[$i]["VRENCARGOAF"]);
                        $encargo[$i]["__is_new"]    = true;
                        array_push($encargosPreenchidos, $encargo[$i]);
                    }
                }
                $this->dataSourceName = "encargoaf";
                $dataSave = new DataSet("encargoaf", (array)$encargosPreenchidos);
                $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($requestSave, $response);
                
                // Insere Impostos
                $requestSave = new DTO\Request\Row($produto, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                $this->insereImpostos($requestSave, $response);
                
                // Salva Justificativa
                if(isset($justif["paramAuditoria"]) && $justif["paramAuditoria"]) {
                    $justif["CDFILDESMOV"] = $dadosAF["CDFILDEST"];
                    $justif["CDFILLANMOV"] = $dadosAF["CDFILLANC"];
                    $justif["NRAUTOMOV"]   = $dadosAF["NRAUTOFORN"];
                    $justif["DTPROGENTMOV"] = null;
                    $this->service->inserirJustificativa($justif);
                }
            }
            $response->addDataSet(new DataSet('inconsistencias', $inconsistencias));
            $response->addDataSet(new DataSet('dadosAF', [$dadosAF]));
            
            if(!empty($anexos)){
                $this->service->salvaAnexoInFileServer($anexos, $dadosAF['CDFILDEST'], $dadosAF['NRAUTOFORN']);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
        $this->service->fechamentoProcAF($dadosAF['CDFILDEST'], $dadosAF['CDFILLANC'], $dadosAF['NRAUTOFORN']);
        $this->service->incluiNFContaPagar($dadosAF['CDFILDEST'], $dadosAF['CDFILLANC'], $dadosAF['NRAUTOFORN']);
    }
    
    public function salvarEdicaoAF(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $dadosAF = $request->getRow();
            
            if (self::exigeLiberacaoAFManual($dadosAF)) {
                $dadosAF["IDSITUATUAAF"] = "1";
            }
            
            $utilizaIndiceFinanceiro = $this->getUtilizaIndiceFinanceiro();
            
            if ($utilizaIndiceFinanceiro) {
                $dadosAF['CDTPINDICEAF'] = $dadosAF['CDTPINDICE'] ?? $dadosAF['CDTPINDICEAF'];
                $dadosAF['IDPERVALORAF'] = $dadosAF['IDPERVALOR'] ?? $dadosAF['IDPERVALORAF'];
                $dadosAF['CDINDICEAF']   = $dadosAF['CDINDICE'] ?? $dadosAF['CDINDICEAF'];
                $dadosAF['DTREFINDAF']   = $dadosAF['DTREFIND'] ?? $dadosAF['DTREFINDAF'];
            } else {
                $dadosAF['CDTPINDICEAF'] = null;
                $dadosAF['IDPERVALORAF'] = null;
                $dadosAF['CDINDICEAF']   = null;
                $dadosAF['DTREFINDAF']   = null;
            }
            
            // Salva AF
            $this->dataSourceName = "autoforne";
            $requestSave = new DTO\Request\Row($dadosAF, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            
            parent::save($requestSave, $response);
            
            // Atualiza valor Total
            $this->service->valorTotalAF($dadosAF);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function finalizaAdicaoAF(DTO\Request $request, DTO\Response $response) {
        try {
            $info     = $request->getDataSet()->getRows()[0];
            $justif   = $request->getDataSet()->getRows()[1];
            $encargo  = $request->getDataSet()->getRows()[2];
            $prazos   = $request->getDataSet()->getRows()[3];
            $mensagem = $request->getDataSet()->getRows()[4];
            
            // Salva Mensagens
            for($i = 0; $i < sizeof($mensagem); $i++) {
                $mensagem[$i]["NRAUTOFORN"] = $info["NRAUTOFORN"];
                $mensagem[$i]["CDFILDEST"]  = $info["CDFILDEST"];
                $mensagem[$i]["CDFILLANC"]  = $info["CDFILLANC"];
            }
            $this->dataSourceName = "mensautf";
            $dataSave = new DataSet("mensautf", (array)$mensagem);
            $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            // Salva Prazo Percentual
            for($i = 0; $i < sizeof($prazos); $i++) {
                $prazos[$i]["NRAUTOFORN"] = $info["NRAUTOFORN"];
                $prazos[$i]["CDFILDEST"]  = $info["CDFILDEST"];
                $prazos[$i]["CDFILLANC"]  = $info["CDFILLANC"];
            }
            $this->salvarPrazoPercentual($request, $response, $prazos);
            
            // Salva Encargos
            $encargosPreenchidos = [];
            for($i = 0; $i < sizeof($encargo); $i++) {
                if($encargo[$i]["VRENCARGOAF"]) {
                    $encargo[$i]["CDFILLANC"]   = $info["CDFILLANC"];
                    $encargo[$i]["CDFILDEST"]   = $info["CDFILDEST"];
                    $encargo[$i]["NRAUTOFORN"]  = $info["NRAUTOFORN"];
                    $encargo[$i]["DTPROGENTAF"] = $info["DTPROGENTAF"];
                    $encargo[$i]["DTENTRENCAF"] = $info["DTPROGENTAF"];
                    $encargo[$i]["VRENCARGOAF"] = $this->valueToFloat($encargo[$i]["VRENCARGOAF"]);
                    $encargo[$i]["__is_new"]    = true;
                    array_push($encargosPreenchidos, $encargo[$i]);
                }
            }
            $this->dataSourceName = "encargoaf";
            $dataSave = new DataSet("encargoaf", (array)$encargosPreenchidos);
            $requestSave = new DTO\Request\DataSet($dataSave, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            parent::save($requestSave, $response);
            
            // Insere Impostos
            $requestSave = new DTO\Request\Row($info, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $this->insereImpostos($requestSave, $response);
            
            // Salva Justificativa
            if(isset($justif["paramAuditoria"]) && $justif["paramAuditoria"]) {
                $justif["CDFILDESMOV"] = $info["CDFILDEST"];
                $justif["CDFILLANMOV"] = $info["CDFILLANC"];
                $justif["NRAUTOMOV"]   = $info["NRAUTOFORN"];
                $justif["DTPROGENTMOV"] = null;
                $this->service->inserirJustificativa($justif);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function excluirAF(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $dadosAF = $request->getRow();
            
            $this->service->excluiNFContaPagar($dadosAF['CDFILDEST'], $dadosAF['CDFILLANC'], $dadosAF['NRAUTOFORN']);
            $this->service->excluirAF($dadosAF);
            $this->service->excluiTodosProdutos($dadosAF);
            
            if (!empty($dadosAF['COOKIE_NAME'])) {
                self::enviarEmailFornecedor($dadosAF);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
     private function dadosFormaPagamento(&$dadosAF) {
        if (isset($dadosAF["IDTPFORPGTFO"])) {
            if ($dadosAF["IDTPFORPGTFO"] == null) {
                $dadosAF["IDTPFORPGTFO"] = null;
                $dadosAF["CDFORMPGTO"]   = null;
                $dadosAF["NRDIAPGFIXAF"] = null;
                $dadosAF["NRPARCPGTOAF"] = null;
                $dadosAF["IDUTDIAPGTAF"] = null;
            } else if ($dadosAF["IDTPFORPGTFO"] == "F") {
                $dadosAF["CDFORMPGTO"]   = null;
            } else if ($dadosAF["IDTPFORPGTFO"] == "V") {
                $dadosAF["NRDIAPGFIXAF"] = null;
                $dadosAF["NRPARCPGTOAF"] = null;
                $dadosAF["IDUTDIAPGTAF"] = null;
            }
        }
    }
    
    /*private function validacoesEntrega($row)
    {   
        if($row["DTPROGENTAF"]) {
            $dtProcessa = self::retornaDtprocessa($row["CDFILDEST"]);
                
            $dtProcessamento = DateTime::createFromFormat("d/m/Y", $dtProcessa);
            $dtLancamento    = DateTime::createFromFormat("d/m/Y", substr($row["DTAUTOFORN"], 0, 10));
            $dtEntrega       = DateTime::createFromFormat("d/m/Y", substr($row["DTPROGENTAF"], 0, 10)); 
    
            if($dtProcessamento && $dtLancamento && $dtEntrega < $dtProcessamento && $dtEntrega < $dtLancamento)
                return 'DTPROGENTAF>DTATUAL&&DTAUTOFORN';
            else if($dtProcessamento && $dtEntrega < $dtProcessamento)
                return 'DTPROGENTAF>DTATUAL';
            else if($dtLancamento && $dtEntrega < $dtLancamento)
                return 'DTPROGENTAF>DTAUTOFORN';
            else if(isset($row["BLOQ_MULT_DATAS"]) && $row["BLOQ_MULT_DATAS"] == "S" && isset($row["NRAUTOFORN"])) {
                if(!$this->service->validaMultiplasDatasEntrega($row))
                    return 'SOMENTE_UMA_DTPROGENTAF';
            }
        } else {
            return 'INVALID_DATE';
        }
    }*/
    
    public function validaDataEntrega(DTO\Request\Row $request, DTO\Response $response) 
    {
        try {
            
            $row = $request->getRow();
            
            self::setCentralDeCompras($row);
            
            //$result = self::validacoesEntrega($row);
            
            if(!empty($result)) {
                $response->addDataSet(self::createMessageDataset($result));
            } else {
                $result = self::verifyProdutoCotacao($row);
                
                if($result === 'FORNECEDOR_SEM_COTACAO') {
                    $dataSet = self::createMessageDataset($result);
                } else if(isset($result)) {
                    $adjust = $this->service->findAdjustments($row);
                    $dataSet = new DataSet('cotacao', [$result, $adjust]);
                } else {
                    $dataSet = self::createMessageDataset(''); 
                }
                
                $response->addDataSet($dataSet);
            }            
    
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function createMessageDataset($message)
    {
        return new DataSet('message', [$message]);
    }
    
    private function validacoesProduto($row) {
        if(!$this->service->produtoAtivo($row))
            return "PRODUTO_IVATIVO";
        else if(!$this->service->produtoAtivoFilial($row))
            return "PRODUTO_IVATIVO_UNIDADE";
        else if(!$this->service->validaProdutoComprador($row))
            return "PRODUTO_NAO_CADASTRADO_COMPRADOR";
        else if(!$this->service->validaProdutoNaoCompradoPelaFilial($row))
            return "PRODUTO_NAO_COMPRADO_UNIDADE";
        else if(!$this->service->validaProdutoPorFornecedor($row))
            return "PRODUTO_NAO_CADASTRADO_FORNECEDOR";
        else if(!$this->service->validaProdutoCompradoPelaFilial($row)) {
            if($this->environment->getCdFilial() != $row['CDFILCENTCMP']) {
                return "PRODUTO_COMPRADO_UNIDADE";
            }/*else if($row['CDFILDEST'] == $this->environment->getCdFilial()) {
                return "PRODUTO_COMPRADO_MATRIZ";
            }*/
        } else {
            $paramProdEstoque = $this->service->getParamUtilProdEstoque($row);
            $produtoEstoque   = $this->service->validaProdutoEstoque($row);
            
            if($paramProdEstoque[0]['IDUTLPROCMPAF'] == 'S' && !empty($produtoEstoque)) {
                return "SOMENTE_PRODUTO_COMPRA";
            }
        }
    }
    
    private function setCentralDeCompras(&$row) {
        if (!isset($row['CDFILCENTCMP']) || ((isset($row['CDFILCENTCMP'])) && empty($row['CDFILCENTCMP'])))
            $row['CDFILCENTCMP'] = $this->cmpUtilities->getCentralCompras()['CDFILCENTCMP'];
    }
    
    public function validaProduto(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            
            self::setCentralDeCompras($row);
            
            $result = self::validacoesProduto($row);
            
            $response->addDataSet(self::createMessageDataset($result));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaProdMarca(DTO\Request\Row $request, DTO\Response $response) 
    {
        try {
            
            $row = $request->getRow();
            
            self::setCentralDeCompras($row);
            
            $result = '';
            
            if(isset($row["BLOQ_MULT_DATAS"]) && $row["BLOQ_MULT_DATAS"] == "S" && isset($row["NRAUTOFORN"]))
                if($this->service->verificaProdutodDuplicado($row))
                    $result = 'PRODUTO_DUPLICADO';
                    
            $response->addDataSet(self::createMessageDataset($result));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function verifyProdutoCotacao($row)
    {
        /**
         * Carrega parâmetro caso não esteja presente na row.
         */
        if (!isset($row['IDCARPRECLAF']) || !isset($row['IDBLOQFORNCOTAF']) || !isset($row['IDUTINDICECOL']))
            $row = $this->getParametrosCotacao($row);
        
        if (!isset($row['CDOPERADORSEMREST']))
            $row = $this->getParamIsOperadorSemRestricao($row);
        
        if($row['IDCARPRECLAF'] == 'S'){
            $cotacao = $this->service->getCotacao($row);
            
            if(!isset($cotacao[0])) {
                
                if($row['IDBLOQFORNCOTAF'] == 'S' && $row['CDOPERADORSEMREST'] <> $row['OPERADOR']){
                    return "FORNECEDOR_SEM_COTACAO";
                }
            } else {
                return $cotacao;
            }
        }
    }
    
    
    /**
     * TO DO 
     * verificar funcionamento
     * 
    */
    public function validateAllProducts(DTO\Request\DataSet $request, DTO\Response $response) 
    {
        try {
            $saveProducts = $request->getDataSet()->getRows();
            
            if(sizeof($saveProducts) > 0){
                
                $parasupri = $this->service->getCdFilcentCmp($saveProducts[0]['CDFILDEST']);
                $centralCompras = $parasupri['CDFILCENTCMP'];
                $operSemRestricao = $parasupri['CDOPERADORSEMREST'];
                $param = $this->service->getParamCotacao($centralCompras);
                $cdcomprador = $this->service->getCdComprador($saveProducts[0]['CDFILLANC'], $saveProducts[0]['CDFILDEST'], $saveProducts[0]['NRAUTOFORN']);
                
                if(sizeof($param) > 0){
                    $idCarPrecAf     = $param[0]['IDCARPRECLAF'];
                    $idBloqFornCotAf = $param[0]['IDBLOQFORNCOTAF'];
                    $cdOperador      = $param[0]['CDOPERADOR'];
                    
                }
                foreach($saveProducts as $row) {
                    $row['CDFILCENTCMP'] = $centralCompras;
                    $row['OPERADOR'] = $cdcomprador;
                    
                    $result = self::validacoesProduto($row);
                    
                    if(!$result)
                        $result = self::verifyProdutoCotacao($row) === 'FORNECEDOR_SEM_COTACAO' ? 'FORNECEDOR_SEM_COTACAO' : '';
                }
                
                $response->addDataSet(self::createMessageDataset($result));
                
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function salvarMensagem(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "mensautf";
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    } 
    
    public function excluirMensagem(DTO\Request\Row $request, DTO\Response $response) {
        try {
            
            $this->dataSourceName = "mensautf";
            parent::delete($request, $response);
            
            $row = $request->getRow();
            $mensagens = $this->service->retornaMensagens($row);
            foreach($mensagens as $mensagem) {
                $this->service->atualizaMensagem($mensagem);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function novoCodigoMensagem(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $novoCodigo = $this->service->novoCodigoMensagem($row) + 1;
            $response->addDataSet(new Dataset("novoCodigo", [$novoCodigo]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    // Adaptado de 'General.js'
    public function valueToFloat($value) {
        if (empty($value)) return 0.0;
        
		if (is_string($value) && $value) {
			if (strpos($value, ',') === false && strpos($value, '.') === false) {
				return floatval($value);
			} else if (strpos($value, ',') > strpos($value, '.')) {
                $value = str_replace('.', '', $value);
                $value = str_replace(',', '.', $value);
				return floatval($value);
			} else if (strpos($value, ',') < strpos($value, '.')) {
			    $value = str_replace(',', '.', $value);
				return floatval($value);
			} else if (strpos($value, ',') === 0 || strpos($value, '.') === 0) {
			    if (strpos($value, ',') === 0) {
			        $value = str_replace(',', '.', $value);
			    }
				return floatval($value);    
			}
		} else {
			return $value;
		}
    }
    
    public function utilizaworkflow(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $utilizaWorkflow = $this->service->utilizaWorkflowController($row);
            
            $response->addDataSet(new DataSet('data', $utilizaWorkflow));
        } catch (\Exception $e) {
            $this->service->excluirAF($row);
            
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function retornaDtprocessa($cdfilial) {
        
        $idutildtproc = $this->service->paramgeral_validacao();
        
        if($idutildtproc == "S") {
            $dtprocessa = $this->service->paramfilial_validacao($cdfilial);
            $dtprocessa = substr($dtprocessa, 0, 10);
        } else {
            $dtprocessa = new DateTime();
            $dtprocessa = $dtprocessa->format('d/m/Y');
        }
        
        return $dtprocessa;
        
    }

    public function retornaParametroCotacao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $centralCompras = $row['CDFILCENTCMP'];
        $param = $this->service->getParamCotacao($centralCompras);
        $dataSet = new DataSet('param', $param);
        $response->addDataSet($dataSet);
    }
        
    public function retornaCotacaoProduto(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $cotacao = $this->service->getCotacao($row);
        if(isset($cotacao[0])) {
            $dataSet = new DataSet('cotacao', $cotacao);
            $response->addDataSet($dataSet);
        } else {
            $response->setError(new Error("Não existe cotação para o produto!", 1));
        }
    }
    
    public function getFormPgtoFornecedor(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $formPgto= $this->service->getFormPgtoFornecedor($row);
        if(isset($formPgto[0])) {
            $dataSet = new DataSet('res', $formPgto);
            $response->addDataSet($dataSet);
        }
    }
    
    public function salvaJustLancAf(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->salvaJustLancAf($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function verificaPrecoCotacaoAF(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $precoCotacaoAF = $this->service->verificaPrecoCotacaoAF($row);
            
            $response->addDataSet(new DataSet('res', $precoCotacaoAF));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function trataParticularidadesPorFornecedorMarca($value)
    {
        switch ($value) {
            case '7':
                return "Escolhido por Fornecedor/Marca Obrigatórios.";
                break;
                
            case '8':
                return "Escolhido por Fornecedor/Marca Preferenciais.";
                break;
                
            case '9':
                return "Escolhido por Fornecedor/Marca Restritivos.";
                break;
        }
    }
    
    public function trataParticularidadesPorFornecedor($value)
    {
        switch ($value) {
            case '4':
                return "Escolhido por Fornecedor Obrigatório.";
                break;
                
            case '5':
                return "Escolhido por Fornecedor Preferencial.";
                break;
                
            case '6':
                return "Escolhido por Fornecedor Restritivo.";
                break;
                
            case 'A':
                return "Escolhido por Acordo.";
                break;
        }
    }
    
    public function trataParticularidadesPorMarca($value)
    {
        switch ($value) {
            case '1':
                return "Escolhido por Marca Obrigatória.";
                break;
                
            case '2':
                return "Escolhido por Marca Preferencial.";
                break;
                
            case '3':
                return "Escolhido por Marca Restritiva.";
                break;
        }
    }
    
    public function trataParticularidadesPorListaPrecoCotacao($value)
    {
        switch ($value) {
            case '1':
                return "Escolhido da Lista de Preços.";
                break;
                
            case '2':
                return "Escolhido da Cotação Fora da Lista.";
                break;
        }
    }
    
    public function getStatusCotacaoAF(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $parametrosAF = $this->service->verificaPrecoCotacaoAF($row);
            
            $mensagens = [];
            
            if (!empty($parametrosAF)) {
                
                $precoCotacaoAF = $parametrosAF['IDMOTPRESCAF'];
                
                for ($i = 0; $i < strlen($precoCotacaoAF); $i++) {
                    switch ($i) {
                        case '0':
                            if ($precoCotacaoAF[$i] == "1") $mensagens[] = $row['MESSAGES'][0];
                            break;
                             
                        case '1':
                            $mensagens[] = self::trataParticularidadesPorFornecedorMarca($precoCotacaoAF[$i]);
                            break;
                            
                        case '2':
                            $mensagens[] = self::trataParticularidadesPorFornecedor($precoCotacaoAF[$i]);
                            break;
                            
                        case '3':
                            $mensagens[] = self::trataParticularidadesPorMarca($precoCotacaoAF[$i]);
                            break;
                            
                        case '4':
                            $mensagens[] = self::trataParticularidadesPorListaPrecoCotacao($precoCotacaoAF[$i]);
                            break;
                    }
                }
                
                if ($parametrosAF['IDTIPOLANITM'] == 'M') {
                    if (empty($mensagens)) {
                        $mensagens[] = $row['MESSAGES'][1];
                        $mensagens[] = str_replace("NRCOTPREITAF", $parametrosAF['NRCOTPREITAF'], $row['MESSAGES'][2]);
                        $mensagens[] = str_replace("NRLISTAITAF", $parametrosAF['NRLISTAITAF'], $row['MESSAGES'][3]);
                        $mensagens[] = $row['MESSAGES'][4];
                    } else {
                        $mensagens[] = str_replace("NRCOTPREITAF", $parametrosAF['NRCOTPREITAF'], $row['MESSAGES'][2]);
                        $mensagens[] = str_replace("NRLISTAITAF", $parametrosAF['NRLISTAITAF'], $row['MESSAGES'][3]);
                        $mensagens[] = $row['MESSAGES'][4];
                    }
                } else {
                    if (empty($mensagens)) {
                        $mensagens[] = $row['MESSAGES'][1];
                        $mensagens[] = str_replace("NRCOTPREITAF", $parametrosAF['NRCOTPREITAF'], $row['MESSAGES'][2]);
                        $mensagens[] = str_replace("NRLISTAITAF", $parametrosAF['NRLISTAITAF'], $row['MESSAGES'][3]);
                    } else {
                        $mensagens[] = str_replace("NRCOTPREITAF", $parametrosAF['NRCOTPREITAF'], $row['MESSAGES'][2]);
                        $mensagens[] = str_replace("NRLISTAITAF", $parametrosAF['NRLISTAITAF'], $row['MESSAGES'][3]);
                    }
                }
                
            } else {
                $mensagens[] = $row['MESSAGES'][5];
            }
            
            $response->addDataSet(new DataSet('res', $mensagens));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function saveAllProduct(DTO\Request $request, DTO\Response $response) {
        try {
            $data = $request->getRow();
            $utilizaIndiceFinanceiro = $this->getUtilizaIndiceFinanceiro();
            $firstRow = true;
            
            foreach ($data['produtos'] as $row) {
                $this->dataSourceName = "itemautf";
            
                if($firstRow) {
                    if(in_array('UPDATE_ALL_ITENS', array_keys($row))) {
                        if($row['BLOQ_MULT_DATAS'] === 'S' && $row['UPDATE_ALL_ITENS'] === 'S') {
                            foreach($this->service->getDiferentItemsAutF($row) as $item) {
                                
                                $this->service->excluiProdutosDiferentes($item);
                                
                                $item['DTPROGENTAF'] = $row['DTPROGENTAF'];
                                $item['__is_new'] = true;
                                
                                $requestNewSave = new DTO\Request\Row($item, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                                parent::save($requestNewSave, $response);
                            }
                        }
                    }
                    $firstRow = false;
                }
                
                $row["IDBAIXITAFS"]     = $row["IDBAIXITAFS"] == 0 ? null : $row["IDBAIXITAFS"];
                $row["IDBAIXITMAF"]     = $row["IDBAIXITMAF"] == 0 ? null : $row["IDBAIXITMAF"];
                $row["QTVOLUITEMAF"]    = $this->valueToFloat($row["QTVOLUITEMAF"]);
                $row["QTPROGENTAF"]     = $this->valueToFloat($row["QTPROGENTAF"]);
                $row["QTPROGENTAFORIG"] = $this->valueToFloat($row["QTPROGENTAFORIG"]);
                $row["VRDESCITMAF"]     = $this->valueToFloat($row["VRDESCITMAF"]);
                $row["PRECOTOTAL"]      = $this->valueToFloat($row["PRECOTOTAL"]);
                $row["VRPERDESAF"]      = $this->valueToFloat($row["VRPERDESAF"]);
                $row["VRPREUNIAF"]      = $this->getPrecoUnitario($data['dadosAF'], $row, $utilizaIndiceFinanceiro);
                $row["VRINDICEITMAF"]   = $this->getPrecoFinanceiro($row, $utilizaIndiceFinanceiro);
                $row["IDATIVO"]         = "S";
                $row["IDBAIXITMFUTURA"] = "X";
                $row["IDITMSEPARADO"]   = "N";
                $row["DTPROGENTAF"]     = $row["DTPROGENTAF"]." 00:00:00";
                
                if(in_array('DTPROGBKP', array_keys($row))) {
                    if($row["DTPROGENTAF"] !== $row['DTPROGBKP']) {
                        $oldRow = $row;
                        $oldRow["DTPROGENTAF"] = $row['DTPROGBKP'];
                        $this->service->excluiProdutosDiferentes($oldRow);
                        $row["__is_new"] = true;
                    }
                }
                
                $requestSave = new DTO\Request\Row($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                parent::save($requestSave, $response);
                
                if(in_array('ATUSOLI', array_keys($row))) {
                    if($row['ATUSOLI'] === 'S') {
                        $row["DSJUSTOPERMOV"] = "Alteração de quantidade de produto na Autorização de Fornecimento.";
                        
                        $this->atuSolicita->updateItemSoli($row, $request, $response);
                    }
                }
                
                $retorno = $this->service->CONSISTE_ORCAMENTO($row);
                
                if(in_array('JUSTIFICATIVAS', array_keys($row))) {
                    foreach($row['JUSTIFICATIVAS'] as $justRow) {
                        $this->service->inserirJustificativa($justRow);
                    }
                }
                
                try {
                    $erroFecha = $this->fechamentoAF->fechaAF($row['CDFILDEST'], $row['CDFILLANC'], $row['NRAUTOFORN'], $row['CDFILCENTCMP']);
                } catch(\Exception $e) {
                    $erroFecha = $e;
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

    /**
     * A data de lançamento pode assumir os seguintes valores em suas respectivas
     * condições:
     * - Data de Processamento da filial selecionada
     *      Condição: Parâmetro '' marcado, filial com data de processamento e parametrizada.
     * - Data atual
     *      Condição: Parâmetro '' desmarcado ou filial com data de processamento
     *                vazia ou filial não parametrizada.
     */
    public function retornaDataLancamento(DTO\Request $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $dataLancamento = $this->service->findDataLancamento($row);
            
            $dataLancamento['DTLANC'] = explode(' ', $dataLancamento['DTLANC'])[0];

            $response->addDataSet(new DataSet('data', $dataLancamento));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getParametrosCotacao($row)
    {
        $params = $this->service->getParamCotacao($row['CDFILCENTCMP'])[0];
        
        $row['IDCARPRECLAF'] = $params['IDCARPRECLAF'];
        $row['IDBLOQFORNCOTAF'] = $params['IDBLOQFORNCOTAF'];
        $row['IDUTINDICECOL'] = $params['IDUTINDICECOL'];
        
        return $row;
    }
    
    public function getParamIsOperadorSemRestricao($row)
    {
        $parasupri = $this->service->getCdFilcentCmp($row['CDFILDEST']);
        
        $row['CDOPERADORSEMREST'] = $parasupri['CDOPERADORSEMREST'];
        
        return $row;
    }
    
    /**
     * Consiste o orçamento do GOP caso a Unidade esteja participando
     * do processo de Workflow.
     */
    public function consisteOrcamentoGop($row)
    {
        $unidadeDest = isset($row['CDFILDEST']) ? $row['CDFILDEST'] : "";
        
        $utilizaWorkflow = $this->service->utilizaWorkflow($unidadeDest);

        if ($utilizaWorkflow == 'N') {
            return;
        }
        
        $unidadeLanc = isset($row['CDFILLANC']) ? $row['CDFILLANC'] : "";
        
        if (isset($row['NRDEMANDA']) || isset($row['NRAUTOFORN'])) {
            if (isset($row['NRDEMANDA'])) {
                $nrdemanda = $row['NRDEMANDA'];
            } else {
                $nrdemanda = $row['NRAUTOFORN'];
            }
        } else {
            $nrdemanda = "";
        }
        
        $retorno = $this->service->CONSISTE_ORCAMENTO_GOP($unidadeDest, $unidadeLanc, $nrdemanda);
        
        return $retorno;
    }
    
    public function verificaStatusAprovacaoWorkflow(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();

            $existsAprovacaoRecusada = $this->service->findAprovacaoRecusada($row);
            
            $data['STATUS'] = empty($existsAprovacaoRecusada) ? 'NAO_PARTICIPOU_WORKFLOW' : $existsAprovacaoRecusada['IDAPROVADO'];

            $response->addDataSet(new DataSet('data', $data));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function getPrecoUnitario($dadosAF, $produto, $utilizaIndiceFinanceiro)
    {
        if ($utilizaIndiceFinanceiro == true && $dadosAF['CDINDICEAF'] != null) {
            $valorIndice = $dadosAF['VRINDICE'] ?? $dadosAF['VRINDICEAF'];
            
            return $this->valueToFloat($produto['VRINDICEITMAF']) * $this->valueToFloat($valorIndice);
        }
        
        return $this->valueToFloat($produto['VRPREUNIAF']);
    }
    
    private function getPrecoFinanceiro($produto, $utilizaIndiceFinanceiro)
    {
        if ($utilizaIndiceFinanceiro) {
            return $this->valueToFloat($produto['VRINDICEITMAF']);
        }
        
        return null;
    }
    
    /**
     * [INÍCIO] ANEXO
     */
    
    /**
     * Adição da AF
     */
    /**
     * Salva Anexo da Autorização de Fornecimento antes de salvar no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function salvaAnexoLocalAF(DTO\Request\Row $row, DTO\Response $response)
    {
        try {
            
            $row  = $row->getRow();
            
            $id = $this->service->salvaAnexoLocal($row);
            
            $response->addDataSet(new DataSet('data', [ 'NRANEXOLANCAF' => $id ]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo da Autorização de Fornecimento antes de salvar no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function deletaAnexoLocalAF(DTO\Request\DataSet $dataset, DTO\Response $response)
    {
        try {
            
            $rows = $dataset->getDataSet()->getRows();
            
            $this->service->deletaAnexoLocal($rows);
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Visualização da AF
     */
     
    /**
     * Salva Anexo da Autorização de Fornecimento no file Server
     * @param Row $row
     * @param Response $response
     */
    public function salvaAnexoGridAF(DTO\Request\Row $row, DTO\Response $response)
    {
        try {
            $row  = $row->getRow();
            
            $this->service->salvaAnexoGrid($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta Anexo da Autorização de Fornecimento e seu arquivo no FileServer
     * @param Row $row
     * @param Response $response
     */
    public function deleteAnexoGridAF(DTO\Request\DataSet $dataset, DTO\Response $response)
    {
        try {
            $rows = $dataset->getDataSet()->getRows();
            
            $this->service->deleteAnexoGrid($rows);
            // $this->service->deletaAnexoLocal($row);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Busca a url do FileServer
     * @param Row $row
     * @param Response $response
     */
    public function getFileServerUrlAF(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $fileName = $this->service->getFileServerUrl();
            
            $response->addDataset(new DataSet('sucess', array("fileName" => $fileName)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * [FIM] ANEXO
     */
     
    public function getAdjustments(DTO\Request\Row $request, DTO\Response $response) 
    {
        try {
            
            $row = $request->getRow();
            $result = $this->service->findAdjustments($row);
            $response->addDataSet(new DataSet('adjustments', $result ? $result : []));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function retornaAFEmAberto(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            
            $this->service->beginTransaction();
            
            $this->service->insMovOpAFAberto($row);
            $this->service->insMovOpAFPar($row);
            
            $this->service->updReAfAbertoPar($row);
            $this->service->updReAfAberto($row);
            
            
            $inConta_Items = $inConta_Null = $inConta_1 = $inConta_2 = $inConta_3 = 0;
            
            $count = $this->service->getCountBaixaItm($row);
            if(is_array($count) && count($count)) {
                $inConta_Items = $count[0]['NRCOUNT'];
                $inConta_Null = $count[0]['NRABERTO'];
                $inConta_1 = $count[0]['NRPARCIAL'];
                $inConta_2 = $count[0]['NRTOTAL'];
                $inConta_3 = $count[0]['NRMANUAL'];
            } 
            
            if($inConta_Items === 0) {
                $stBaixaAF = '0'; //Caso não exista itens a AF
            } else {
                if((($inConta_Null >= 0) && ($inConta_Null < $inConta_Items) || (($inConta_Items === $inConta_Null + $inConta_2 +  $inConta_3) && (($inConta_Null >= 0) || ($inConta_1 > 0))) || ($inConta_1 === $inConta_Items))
                    && ($inConta_Null < $inConta_Items) && ($inConta_2 < $inConta_Items) && ($inConta_3 < $inConta_Items)) {
                    
                    $stBaixaAF = '1'; //Se todos os itens não estiverem baixados, seja com Baixa Manual ou Total
                } else {
                    if($inConta_2 === $inConta_Items) {
                       $stBaixaAF = '2'; // Se todos os itens estiverem com Baixa Total
                    } else {
                        if($inConta_3 === $inConta_Items) {
                          $stBaixaAF = '3'; // Se todos os itens estiverem com baixa manual
                        } else {
                            if($inConta_Null === 0 && $inConta_1 === 0) {
                                $stBaixaAF = '3';
                            } else {
                                $stBaixaAF = '';
                            }
                        }
                    }
                }
            }
            
            $this->service->updAtuBaixaAF($row, $stBaixaAF);
            $this->service->AJUSTE_CONTPREVISTA($row);
            
            $resultMessage = empty($stBaixaAF) ? 'SUCCESS_OPERATION' : 'EM_ABERTO_ERROR';
            
            $this->service->commit();
            
            $response->addDataSet(new DataSet('baixaAF', [$stBaixaAF]));

        } catch (\Exception $e) {
            $this->service->rollBack();
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function inconsistenciaAF(DTO\Request\Filter $request, DTO\Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $dataset = $this->crudService->getFilteredDataSet("inconsistencia", $conditions)->getRows();
            
            $response->addDataSet(new DataSet('dados', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function validaSegMercadoProdFili(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $cdSegMerc = $this->service->getCdSegMercFilial($row["CDFILDEST"]);
            
            if($cdSegMerc) {
                $res = $this->service->verificaSegMercado($row);
                
                if(empty($res) || count($res) == 0) {
                    throw new \Exception("PRODUTO_NOT_SEGMERCADO", 500);
                }
            }
            
            $res1 = $this->service->verificaProdFiliAtivo($row);
            
            if(empty($res1) || count($res1) == 0) {
                throw new \Exception("PRODUTO_NOT_PRODFILI_ATIVO", 500);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}