<?php
namespace Geral\Entities;

use Geral\Util\Ambiente;
use \Setforms2\Shared\Registry;
use \Geral\Util\DateUtil;

/**
 *
 * @method \Geral\Entities\Paramfilial findOneByCdfilialAndNrorg($cdfilial, $nrorg) static
 */
class Paramfilial extends Base\Paramfilial
{

    /**
     * IDUTCTRLFIS: Confirma que a filial utiliza Controle fiscal.
     */
    const UTILIZA_CONTROLE_FISCAL = "S";
    /**
     * IDUTCTRLFIS_SIM: Nega o fato de que a filial utiliza controle fiscal.
     */
    const NAO_UTILIZA_CONTROLE_FISCAL = "N";

    /**
     * IDINVROTDIA = S
     */
    const UTILIZA_INVENTARIO_ROTATIVO_DIARIO     = "S";
    /**
     * IDINVROTDIA = N
     */
    const NAO_UTILIZA_INVENTARIO_ROTATIVO_DIARIO = "N";

    const IDPERFORNECO_SIM = "S";
    const IDPERFORNECO_NAO = "N";

    /**
     * IDUTILALMOX = S
     */
    const UTILIZA_ESTOQUE     = "S";
    /**
     * IDUTILALMOX = N
     */
    const NAO_UTILIZA_ESTOQUE = "N";

    /**
     * IDUTILLCESTQ = S.
     */
    const UTILIZA_LOCAL_ESTOQUE     = "S";
    /**
     * IDUTILLCESTQ = N.
     */
    const NAO_UTILIZA_LOCAL_ESTOQUE = "N";


    /**
     * Verifica se determinada filial utiliza Controle fiscal.
     *
     * @return bool
     */
    public function utilizaControleFiscal() {
         return $this->getIdutctrlfis() === self::UTILIZA_CONTROLE_FISCAL;
    }

    /**
     * @param $id
     */
    public function atualizaFechamentoDia($id){
        /** @var $obj \Geral\Entities\Paramfilial */
        $this->setIdinvrotok($id);
        $this->persist();
    }


    /**
     * @param $data
     */
    public function atualizaDataProcessamento($data){
        /** @var $obj \Geral\Entities\Paramfilial */
        $this->setDtprocessa($data);
        $this->persist();
    }

    public function getDataProcessamento(){
		return $this->getDtprocessa() ? : DateUtil::getDataAtual();
    }

    public static function insere($cdfilial, $idctrlestq, $nrorg)
    {
        $connection = Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            $parfili = new Paramfilial();
            $parfili->setCdfilial($cdfilial);
            $parfili->setIdctrlestq($idctrlestq);
            $parfili->setNrorg($nrorg);
            $parfili->setNrorginclusao($nrorg);
            $parfili->setNrorgultatu($nrorg);
            $parfili->setDtinclusao(DateUtil::getDataAtual());
            $parfili->setDtultatu(DateUtil::getDataAtual());
            $parfili->setCdoperinclusao(Ambiente::getCdOperador());
            $parfili->setCdoperultatu(Ambiente::getCdOperador());
            $parfili->persist();
            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
}