function Cmp04200_liberacao_af_inconsistente_fora_do_limiteController(Cmp04200_LiberacaoAfInconsistenteForaLimite_Service, General, Utilities, ScreenService, localStorageEngine, templateManager, ZhFilterPreferences, MessageService) {
    const ServiceLiberacaoAF = Cmp04200_LiberacaoAfInconsistenteForaLimite_Service;
    
    /******************
     * Funções Gerais *
     ******************/
    
    function showErrorMessage(response) {
        let errorMessage = 'Timeout';
                
        if (response.data) {
            errorMessage = response.data.error;
        }
        
        ScreenService.showMessage(errorMessage, 'error');
        
        if (!Util.isEmptyOrBlank(messageManager.processes)) {
            messageManager.unsubscribeAll();
        }
    }
    
    /******************
     * Grid Principal *
     ******************/
    
    function checksSupplyAuthorizationSituation(row, situation) {
        return row.SITUACAOAF.includes(situation);
    }
    
    function getSituationLabel(filterRow) {
        if (checksSupplyAuthorizationSituation(filterRow, 'IDAFFORALIM') &&
            checksSupplyAuthorizationSituation(filterRow, 'IDAFINCONS')) {
            return ScreenService.i18n('AFs Fora do Limite e AFs Inconsistentes');
        }
        
        if (checksSupplyAuthorizationSituation(filterRow, 'IDAFINCONS')) {
            return ScreenService.i18n('AFs Inconsistentes');
        }
        
        return ScreenService.i18n('AFs Fora do Limite');
    }
    
    this.setStripe = function(widget){
        const filterRow = widget.appliedFilter.FILTER_ROW;
        const stripeRow = widget.container.getWidget('stripe').currentRow;
        const widgetGridPrincipal = templateManager.container.getWidget('cmp04200_grid');
        
        stripeRow.SITUACAO = getSituationLabel(filterRow);
        stripeRow.UNIDEST = `Unidade Destino: ${filterRow.NMFILIAL}`;
        stripeRow.COMPRADOR = `Comprador: ${filterRow.NMCOMPRADOR}`;
        stripeRow.GERENTE = `Gerente: ${filterRow.NMGERENTE}`;
        stripeRow.DATA = `Período de entrega: ${filterRow.PERIODO.start} a ${filterRow.PERIODO.end}`;
        
        widgetGridPrincipal.reload();
        widgetGridPrincipal.activate();
    };
    
    this.gridOnEnterOnce = function(widget) {
        const filterRow = {
            CDFILDEST: getLocalVar('CDFILIAL'),
            SITUACAOAF: [ "IDAFINCONS", "IDAFFORALIM" ],
            PERIODO: {
                start: moment().format('L'),
                end: moment().format('L'),
                formattedStart: moment().format('L'),
                formattedEnd: moment().format('L'),
            },
        };
        
        Utilities.startFilter(widget, filterRow);
    };
    
    /**************
     * Liberar AF *
     **************/

    this.liberarAutorizacaoFornecimento = function(widget) {
        const codigoGerente = widget.appliedFilter.FILTER_ROW.CDGERENTE;
        const adicionarGerenteNaRow = row => row.CDGERENTE = codigoGerente;
        const rows = widget.getCheckedRows();
        
        rows.forEach(adicionarGerenteNaRow);
        
        ScreenService.confirmMessage('Confirma a liberação da(s) AF(s) selecionada(s)?', 'question', function(success) {
            ServiceLiberacaoAF.verificaUtilizaWorkflow(rows[0]).then(function(response) {
                const utilizaWorkflow = response.dataset.data.IDUTLWORKFLOW == 'S';
                const processes = [
                    'emailLiberacao'
                ];
                
                if (utilizaWorkflow) {
                    processes.push('workflow');
                }
                
                registerProcesses(processes);
                
                ServiceLiberacaoAF.liberarAutorizacaoFornecimento(rows).then(function(response) {
                    if (utilizaWorkflow) {
                        const message = response.messages ? response.messages.shift().message : '';
                        
                        messageManager.notify('workflow', message);
                    }
                    
                    envioDeEmailParaResponsaveisPelaAf(widget);
                }).catch(showErrorMessage);
            }).catch(showErrorMessage);
        });
    };
    
    /**
     * Realiza uma chamada no backend para a função que dispara
     * emails para os responsáveis da AF  que está sendo liberada.
     *
     * @param {object} widget widget de produtos ("WDGT_PRODUTOS_AF").
     */
    //notifiesAfResponsibleAboutProductEditing (NOME ANTIGO)

    function envioDeEmailParaResponsaveisPelaAf(widget) {
        const filterRow = widget.appliedFilter.FILTER_ROW;
        const rows = widget.getCheckedRows();
        
        rows[0].COD_COMPRADOR = filterRow.CDCOMPRADOR;
        
        ServiceLiberacaoAF.enviarEmailAosResponsaveisPelaAF(rows).then(function(success) {
            messageManager.notify('emailLiberacao', 'Notificação de e-mail de liberação de AF enviada ao responśavel.');
        }, function(error) {
            messageManager.notify('emailLiberacao', 'Não foi possível enviar a notificação de e-mail de liberação de AF ao responsável.');
        });
        
        widget.reload();
    }
    
    /**********
     * Filter *
     **********/
    
    this.filtrar = function(args) {
        const widget = args.owner;
        
        if (!widget.SENHA_GERENTE) {
            args.e.preventDefault();
            MessageService.showMessageByCode("INVALID_PASSWORD");
        }
        
        const filterRow = args.e.preference.FILTER_ROW;
            
        if (typeof(filterRow) === 'object'){
            args.e.preference.CUSTOM_FILTER_ROW = {
                "CDFILDEST": filterRow.CDFILDEST,
                "CDCOMPRADOR": filterRow.CDCOMPRADOR,
                "CDGERENTE": filterRow.CDGERENTE,
                "DTINICIAL": filterRow.PERIODO.start,
                "DTFINAL": filterRow.PERIODO.end,
                "IDAFFORALIM": filterRow.SITUACAOAF.includes("IDAFFORALIM") === true ? "S" : "",
                "IDAFINCONS": filterRow.SITUACAOAF.includes("IDAFINCONS") === true ? "S" : "",
                "NRORG": getLocalVar("NRORG")
            };
        }
        
        return ServiceLiberacaoAF.considerarAFsTodosCompradoresNaValidacaoDoLimiteGerente(filterRow).then(function(callback) {
            if (callback.dataset.IDAFCOMPLIMGER[0] === "S") {
                widget.dataSource.name = "cmp_fornecimento_autorizacaoFornecimento#/crud/af_fora_limite_inconsistente_2";
            } else {
                widget.dataSource.name = "cmp_fornecimento_autorizacaoFornecimento#/crud/af_fora_limite_inconsistente";
            }
            
            widget.currentRow.CDGERENTE = filterRow.CDGERENTE;
            widget.reload();
        }, function(error) {
            showErrorMessage(error);
        });
    };
    
    /**************************
     * Filter - Manager Field *
     **************************/
    
    function filtraGerente(field) {
        field.dataSourceFilter = [
            {
                name: "CDOPERGERE",
                operator: "=",
                value: "T"
            },
            {
                name: "CDOPERCOMP",
                operator: "=",
                value: field.widget.currentRow.CDCOMPRADOR
            },
            {
                name: "NRORG",
                operator: "=",
                value: getLocalVar("NRORG")
            }
        ];
    }
    
    function cleanManagerData(widget) {
        const passwordFields = ['NMGERENTE', 'CDGERENTE'];
        const isPasswordField = field => passwordFields.includes(field.name);
        
        widget.fields.filter(isPasswordField).forEach(field => field.clearValue());
    }
    
    this.habilitaGerente = function(field) {
        const managerField = field.widget.getField("NMGERENTE");
        
        cleanManagerData(field.widget);
        
        managerField.readOnly = !field.value();
        
        filtraGerente(managerField);
    };
    
    /**************************
     * Manager Authentication *
     **************************/
    
    this.openManagerAuthenticationWidget = function(widget) {
        if (!Util.isEmptyOrBlank(widget.currentRow.NMGERENTE)) {
            const widgetAutenticar = widget.container.getWidget('widget_autenticar');
            const widgetGridPrincipal = widget.container.getWidget('cmp04200_grid');
            
            widgetAutenticar.currentRow.CDGERENTE = widget.currentRow.CDGERENTE;
            widgetAutenticar.currentRow.USER = widget.currentRow.NMGERENTE; 
            widgetAutenticar.currentRow.SENHA = "";
            
            ScreenService.openPopup(widgetAutenticar);
            
            widgetGridPrincipal.activate();    
        }
    };
    
    this.cancelarSenhaGerente = function(args, widget) {
        const widgetGridPrincipal = widget.container.getWidget('cmp04200_grid');
        
        widgetGridPrincipal.SENHA_GERENTE = false;
        ScreenService.closePopup();
        
        cleanManagerData(widgetGridPrincipal);
        
        widgetGridPrincipal.activate(); 
    };
    
    this.confirmarSenhaGerente = function(args) {
        const widget = args.owner.widget;
        
        if (widget.isValid()) {
            const widgetGridPrincipal = widget.container.getWidget('cmp04200_grid');
            
            ServiceLiberacaoAF.confirmaSenhaGerente(widget.currentRow).then(function() {
                widgetGridPrincipal.SENHA_GERENTE = true;
                ScreenService.closePopup();
            }, function(err) {
                widgetGridPrincipal.SENHA_GERENTE = false;
                ScreenService.showMessage(err.data.error);
            });
        }    
    };
    
    /*******************
     * Message Manager *
     *******************/
    
    const messageManager = {
        processes: [],
        subscribe: function(processName) {
            this.processes.push({
                name: processName,
                hasNotified: false,
                messages: [],
            });
        },
        unsubscribe: function(processName) {
            this.processes = this.processes.filter(process => process.name != processName);
        },
        unsubscribeAll: function() {
            this.processes = [];
        },
        notify: function(processName, message) {
            const process = this.processes.filter(process => process.name == processName).shift();
            
            process.hasNotified = true;
            
            if (message) {
                process.messages.push(message);
            }
            
            if (this.processes.every(process => process.hasNotified)) {
                this.showMessages();
                this.unsubscribeAll();
            }
        },
        showMessages: function() {
            const messages = this.processes.filter(process => !Util.isEmptyOrBlank(process.messages)).map(process => process.messages.join(' '));
            const separator = messages.length > 1 ? '<br><br>' : '';
            const message = messages.join(separator);
            
            ScreenService.showMessage(message);
        }
    };
    
    function registerProcesses(processes) {
        processes.forEach(processName => {
            messageManager.subscribe(processName);
        });
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Cmp04200_liberacao_af_inconsistente_fora_do_limiteController', Cmp04200_liberacao_af_inconsistente_fora_do_limiteController);
});